@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
- * Copyright (c) 2009 Thomas Schmitt
+ * Copyright (c) 2009 - 2012 Thomas Schmitt
  * 
  * This file is part of the libisofs project; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License version 2 
@@ -49,6 +49,10 @@ struct Iso_Image
     char *copyright_file_id;
     char *abstract_file_id;
     char *biblio_file_id;
+    char *creation_time;
+    char *modification_time;
+    char *expiration_time;
+    char *effective_time;
     
     /* el-torito boot catalog */
     struct el_torito_boot_catalog *bootcat;
@@ -180,6 +184,13 @@ struct Iso_Image
      */
     int generator_is_running;
 
+    /* Pointers to directories or files which shall be get a HFS+ blessing.
+     * libisofs/hfsplus.c et.al. will compare these pointers
+     * with the ->node pointer of Ecma119Nodes.
+     * See libisofs.h
+     */
+    IsoNode *hfsplus_blessed[ISO_HFSPLUS_BLESS_MAX];
+
 };
 
 
@@ -227,4 +238,8 @@ int iso_image_set_checksums(IsoImage *image, char *checksum_array,
                             uint32_t idx_count, int flag);
 
 
+int iso_image_set_pvd_times(IsoImage *image,
+                            char *creation_time, char *modification_time,
+                            char *expiration_time, char *effective_time);
+
 #endif /*LIBISO_IMAGE_H_*/
