@@ -133,6 +133,9 @@ int Xorriso_lst_detach_text(struct Xorriso_lsT *entry, int flag);
 int Xorriso_lst_get_last(struct Xorriso_lsT *entry, struct Xorriso_lsT **last,
                          int flag);
 
+int Xorriso_lst_concat(struct Xorriso_lsT *first, struct Xorriso_lsT *second,
+                       int flag);
+
 
 int Exclusions_new(struct ExclusionS **o, int flag);
 
