@@ -25,6 +25,7 @@
 #include <sys/time.h>
 #include <time.h>
 #include <errno.h>
+#include <pthread.h>
 
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
@@ -133,6 +134,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  strcpy(m->list_delimiter, "--");
  m->ino_behavior= 7;
  m->iso_level= 3;
+ m->iso_level_is_default= 1;
  m->do_joliet= 0;
  m->do_hfsplus= 0;
  m->do_fat= 0;
@@ -147,6 +149,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->scdbackup_tag_written[0]= 0;
  m->scdbackup_tag_listname[0]= 0;
  m->relax_compliance= 0;
+ m->allow_dir_id_ext_dflt= 1;
  m->rr_reloc_dir[0]= 0;
  m->rr_reloc_flags= 1;
  m->untranslated_name_len= 0;
@@ -240,6 +243,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->boot_image_bin_path[0]= 0;
  m->boot_image_bin_form[0]= 0;
  m->boot_image_emul= 0;
+ m->boot_emul_default= 1;
  m->boot_image_load_size= 4 * 512;     /* hearsay out of libisofs/demo/iso.c */
  memset(m->boot_id_string, 0, sizeof(m->boot_id_string));
  memset(m->boot_selection_crit, 0, sizeof(m->boot_selection_crit));
@@ -317,6 +321,20 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
    m->info_msglists[i]= NULL;
    m->msglist_flags[i]= 0;
  }
+ m->lib_msg_queue_lock_ini= 0;
+ m->result_msglists_lock_ini= 0;
+ m->write_to_channel_lock_ini= 0;
+ m->msg_watcher_lock_ini= 0;
+ m->msg_watcher_state= 0;
+ m->msgw_result_handler= NULL;
+ m->msgw_result_handle= NULL;
+ m->msgw_info_handler= NULL;
+ m->msgw_info_handle= NULL;
+ m->msgw_stack_handle= -1;
+ m->msgw_msg_pending= 0;
+ m->msgw_fetch_lock_ini= 0;
+ m->msg_sieve= NULL;
+ m->msg_sieve_disabled= 0;
  m->msglist_stackfill= 0;
  m->status_history_max= Xorriso_status_history_maX;
  m->scsi_log= 0;
@@ -327,6 +345,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  Xorriso__text_to_sev(m->abort_on_text, &m->abort_on_severity, 0);
  m->abort_on_is_default= 1;
  m->problem_status= 0;
+ m->problem_status_lock_ini= 0;
  m->problem_status_text[0]= 0;
  m->errfile_log[0]= 0;
  m->errfile_mode= 0;
@@ -357,6 +376,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->insert_count= 0;
  m->insert_bytes= 0;
  m->error_count= 0;
+ m->launch_frontend_banned= 0;
  m->pacifier_style= 0;
  m->pacifier_interval= 1.0;
  m->pacifier_count= 0;
@@ -433,6 +453,30 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  if(ret <= 0)
    goto failure;
  Xorriso_preparer_string(m, m->preparer_id, 1); /* avoids library calls */
+ ret= pthread_mutex_init(&(m->lib_msg_queue_lock), NULL);
+ if(ret != 0)
+   goto failure;
+ m->lib_msg_queue_lock_ini= 1;
+ ret= pthread_mutex_init(&(m->result_msglists_lock), NULL);
+ if(ret != 0)
+   goto failure;
+ m->result_msglists_lock_ini= 1;
+ ret= pthread_mutex_init(&(m->write_to_channel_lock), NULL);
+ if(ret != 0)
+   goto failure;
+ m->result_msglists_lock_ini= 1;
+ ret= pthread_mutex_init(&(m->problem_status_lock), NULL);
+ if(ret != 0)
+   goto failure;
+ m->problem_status_lock_ini= 1;
+ ret= pthread_mutex_init(&(m->msg_watcher_lock), NULL);
+ if(ret != 0)
+   goto failure;
+ m->msg_watcher_lock_ini= 1;
+ ret= pthread_mutex_init(&(m->msgw_fetch_lock), NULL);
+ if(ret != 0)
+   goto failure;
+ m->msgw_fetch_lock_ini= 1;
 
  if(leafname != NULL)
    free(leafname);
@@ -516,6 +560,20 @@ int Xorriso_destroy(struct XorrisO **xorriso, int flag)
 
  Xorriso_detach_libraries(m, flag&1);
 
+ if(m->lib_msg_queue_lock_ini)
+   pthread_mutex_destroy(&(m->lib_msg_queue_lock));
+ if(m->result_msglists_lock_ini)
+   pthread_mutex_destroy(&(m->result_msglists_lock));
+ if(m->write_to_channel_lock_ini)
+   pthread_mutex_destroy(&(m->write_to_channel_lock));
+ if(m->problem_status_lock_ini)
+   pthread_mutex_destroy(&(m->problem_status_lock));
+ if(m->msg_watcher_lock_ini)
+   pthread_mutex_destroy(&(m->msg_watcher_lock));
+ if(m->msgw_fetch_lock_ini)
+   pthread_mutex_destroy(&(m->msgw_fetch_lock));
+ Xorriso_sieve_dispose(m, 0);
+ 
  free((char *) m);
  *xorriso= NULL;
  return(1);
