@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -278,7 +278,10 @@ no_volunteer:;
      ret= Xorriso_option_fs(xorriso, cpt, 0);
      if(ret<=0)
        goto ex;
-   } else if(strcmp(argv[i], "--grow_overwriteable_iso")==0) {
+   } else if(strcmp(argv[i], "--grow_overwriteable_iso")==0 ||
+             strcmp(argv[i], "--grow_overwritable_iso")==0 ||
+             strcmp(argv[i], "--grow_overriteable_iso")==0
+             ) { /* (A history of typos) */
                                                      /* intentional: argv[i] */
      do_grow= 1;
    } else if(strcmp(argpt, "-help")==0) {
@@ -394,7 +397,7 @@ no_volunteer:;
  /* Perform actions */
  Xorriso_option_report_about(xorriso, "NOTE", 0);
  if(do_version) {
-   sprintf(xorriso->result_line, "Cdrecord 2.01-Emulation Copyright (C) 2012 see libburnia-project.org xorriso\n");
+   sprintf(xorriso->result_line, "Cdrecord 2.01-Emulation Copyright (C) 2013 see libburnia-project.org xorriso\n");
    Xorriso_result(xorriso, 1);
    Xorriso_option_version(xorriso, 0);
  }
@@ -586,7 +589,8 @@ int Xorriso_genisofs_count_args(struct XorrisO *xorriso, int argc, char **argv,
  static char arg1_options[][41]= {
     "-abstract", "-A", "-appid", "-biblio", "-b", "-B", "-boot-load-seg",
     "-boot-load-size", "-C", "-c", "-check-session", "-copyright",
-    "-dir-mode", "-file-mode", "-G", "-gid", "-hide", "-hide-list",
+    "-dir-mode", "-eltorito-id", "-eltorito-selcrit",
+    "-file-mode", "-G", "-gid", "-hide", "-hide-list",
     "-hidden", "-hidden-list", "-hide-joliet", "-hide-joliet-list",
     "-hide-hfsplus", "-hide-hfsplus-list",
     "-hide-udf", "-hide-udf-list", "-input-charset", "-output-charset",
@@ -699,20 +703,13 @@ no_volunteer:;
 }
 
 
-int Xorriso_genisofs_add_boot(struct XorrisO *xorriso, char *whom,
-                              int *option_b, int *emul_boot, int flag)
+int Xorriso_genisofs_add_boot(struct XorrisO *xorriso, int flag)
 {
  int ret;
 
- if(*option_b)
-   xorriso->boot_image_emul= *emul_boot;
- else
-   xorriso->boot_image_emul= 0;
  ret= Xorriso_attach_boot_image(xorriso, 0);
  if(ret <= 0)
    xorriso->boot_image_bin_path[0]= 0;
- *option_b= 0;
- *emul_boot= 2;
  return(ret);
 }
 
@@ -820,6 +817,8 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  -hard-disk-boot             Boot image is a hard disk image",
 "  -no-emul-boot               Boot image is 'no emulation' image",
 "  -boot-info-table            Patch boot image with info table",
+"  -eltorito-id ID             Set El Torito Id String",
+"  -eltorito-selcrit HEXBYTES  Set El Torito Selection Criteria",
 "  -isohybrid-gpt-basdat       Mark El Torito boot image as Basic Data in GPT",
 "  -isohybrid-gpt-hfsplus      Mark El Torito boot image as HFS+ in GPT",
 "  -isohybrid-apm-hfsplus      Mark El Torito boot image as HFS+ in APM",
@@ -829,7 +828,7 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  -partition_offset LBA       Make image mountable by first partition, too",
 "  -partition_sec_hd NUMBER    Define number of sectors per head",
 "  -partition_hd_cyl NUMBER    Define number of heads per cylinder",
-"  -partition_cyl_align MODE   Control cylinder alignment: off, on, auto",
+"  -partition_cyl_align MODE   Control cylinder alignment: off, on, auto, all",
 "  -mips-boot FILE             Set mips boot image name (relative to image root)",
 "  -mipsel-boot FILE           Set mipsel boot image name (relative to image root)",
 "  -B FILES, -sparc-boot FILES Set sparc boot image names",
@@ -1089,9 +1088,10 @@ int Xorriso_genisofs(struct XorrisO *xorriso, char *whom,
                      int argc, char **argv, int flag)
 {
  int ret, i, j, was_path= 0, was_other_option= 0, mem_graft_points, mem;
- int do_print_size= 0, fd, idx, iso_level= 1, emul_boot= 2;
- int option_b= 0, was_failure= 0, fret, lower_r= 0, zero= 0;
- int dir_mode= -1, file_mode= -1, count, partition_number, allow_dir_id_ext= 1;
+ int do_print_size= 0, fd, idx, iso_level= -1;
+ int was_failure= 0, fret, lower_r= 0, zero= 0;
+ int dir_mode= -1, file_mode= -1, count, partition_number;
+ int allow_dir_id_ext= -1;
  int root_seen= 0, do_md5_mem, option_d= 0, arg_count;
  mode_t mode_and, mode_or;
  int with_boot_image= 0, with_cat_path= 0, with_emul_toc= 0;
@@ -1169,7 +1169,7 @@ int Xorriso_genisofs(struct XorrisO *xorriso, char *whom,
  continue;
    if(strcmp(argpt, "-version")==0) {
      sprintf(xorriso->result_line,
-"mkisofs 2.01-Emulation Copyright (C) 2012 see libburnia-project.org xorriso\n"
+"mkisofs 2.01-Emulation Copyright (C) 2013 see libburnia-project.org xorriso\n"
             );
      fd= xorriso->dev_fd_1;
      if(fd<0)
@@ -1692,6 +1692,8 @@ not_enough_args:;
              strcmp(argpt, "-c") == 0 ||
              strcmp(argpt, "-eltorito-catalog") == 0 ||
              strcmp(argpt, "-boot-load-size") == 0 ||
+             strcmp(argpt, "-eltorito-id") == 0 ||
+             strcmp(argpt, "-eltorito-selcrit") == 0 ||
              strcmp(argpt, "--embedded-boot")==0 ||
              strcmp(argpt, "-generic-boot")==0 ||
              strcmp(argpt, "-G") == 0 ||
@@ -2067,10 +2069,14 @@ rr_reloc_dir:;
 
      /* Enforce odd mkisofs defaults on first pathspec */
      xorriso->pacifier_style= 1;
+     if(xorriso->allow_dir_id_ext_dflt && allow_dir_id_ext < 0)
+       allow_dir_id_ext= 1;
      if(allow_dir_id_ext == 1) {
        Xorriso_relax_compliance(xorriso, "allow_dir_id_ext", 0);
        allow_dir_id_ext= 2;
      }
+     if(xorriso->iso_level_is_default && iso_level < 0)
+       iso_level= 1;
      if(iso_level >= 1 && iso_level <= 3) {
        sprintf(sfe, "iso_9660_level=%d", iso_level);
        Xorriso_relax_compliance(xorriso, sfe, 0);
@@ -2164,9 +2170,11 @@ problem_handler_2:;
    else
      argpt= argv[i];
    if(strcmp(argpt, "-no-emul-boot")==0) {
-     emul_boot= xorriso->boot_image_emul= 0;
+     xorriso->boot_image_emul= 0;
+     xorriso->boot_emul_default= 0;
    } else if(strcmp(argpt, "-hard-disk-boot")==0) {
-     emul_boot= xorriso->boot_image_emul= 1;
+     xorriso->boot_image_emul= 1;
+     xorriso->boot_emul_default= 0;
    } else if(strcmp(argpt, "-boot-info-table")==0) {
      xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~2) | 1;
    } else if(strcmp(argpt, "-b") == 0 ||
@@ -2176,14 +2184,14 @@ problem_handler_2:;
      i++;
      if(strcmp(argpt, "--efi-boot") == 0) {
        if(xorriso->boot_image_bin_path[0]) {
-         ret= Xorriso_genisofs_add_boot(xorriso, whom,
-                                        &option_b, &emul_boot, 0);
+         ret= Xorriso_genisofs_add_boot(xorriso, 0);
          if(ret <= 0)
            goto problem_handler_boot;
        }
        boot_path= xorriso->boot_image_bin_path;
        xorriso->boot_efi_default= 1;
-       emul_boot= xorriso->boot_image_emul= 0;
+       xorriso->boot_image_emul= 0;
+       xorriso->boot_emul_default= 0;
 
      } else {
        boot_path= xorriso->boot_image_bin_path; 
@@ -2192,8 +2200,8 @@ problem_handler_2:;
        else
          xorriso->boot_platform_id= 0x00;
        xorriso->boot_efi_default= 0;
-       option_b= 1;
-       xorriso->boot_image_emul= emul_boot;
+       if(xorriso->boot_emul_default)
+         xorriso->boot_image_emul= 2;
      }
      boot_path[0]= 0;
      if(argv[i][0] != '/')
@@ -2202,10 +2210,7 @@ problem_handler_2:;
      if(ret <= 0)
        goto ex;
      if(xorriso->boot_efi_default && xorriso->boot_image_bin_path[0]) {
-       option_b= 0;
-       emul_boot= 0;
-       ret= Xorriso_genisofs_add_boot(xorriso, whom,
-                                      &option_b, &emul_boot, 0);
+       ret= Xorriso_genisofs_add_boot(xorriso, 0);
        if(ret <= 0)
          goto problem_handler_boot;
      }
@@ -2231,6 +2236,18 @@ problem_handler_2:;
      i++;
      sscanf(argv[i], "%d", &ret);
      xorriso->boot_image_load_size= ret * 512;
+   } else if(strcmp(argpt, "-eltorito-id") == 0 ||
+             strcmp(argpt, "-eltorito-selcrit") == 0) {
+     if(i+1>=argc)
+       goto not_enough_args;
+     i++;
+     if(strcmp(argpt, "-eltorito-id") == 0)
+       sprintf(sfe, "id_string=%s", argv[i]);
+     else
+       sprintf(sfe, "sel_crit=%s", argv[i]);
+     ret= Xorriso_option_boot_image(xorriso, "any", sfe, 0);
+     if(ret <= 0)
+       goto problem_handler_boot;
    } else if(strncmp(argpt, "isolinux_mbr=", 13)==0) {
      sprintf(sfe, "isohybrid=%s", argpt + 13);
      ret= Xorriso_option_boot_image(xorriso, "isolinux", sfe, 0);
@@ -2245,8 +2262,7 @@ problem_handler_2:;
    } else if(strcmp(argpt, "-isohybrid-apm-hfsplus") == 0) {
      xorriso->patch_isolinux_image = xorriso->patch_isolinux_image | (1 << 8);
    } else if(strcmp(argpt, "-eltorito-alt-boot")==0) {
-     ret= Xorriso_genisofs_add_boot(xorriso, whom,
-                                    &option_b, &emul_boot, 0);
+     ret= Xorriso_genisofs_add_boot(xorriso, 0);
      if(ret <= 0)
        goto problem_handler_boot;
    } else if(strcmp(argpt, "--embedded-boot")==0 ||
@@ -2338,11 +2354,6 @@ problem_handler_boot:;
  }
  if(with_boot_image && with_cat_path == 0)
    strcpy(xorriso->boot_image_cat_path, "/boot.catalog");
- if(xorriso->boot_image_bin_path[0]) {
-   ret= Xorriso_genisofs_add_boot(xorriso, whom, &option_b, &emul_boot, 0);
-   if(ret <= 0)
-     goto ex;
- }
  /* The boot catalog has to be hidden separately */
  if(xorriso->boot_image_cat_path[0]) {
    ret= Xorriso_path_is_hidden(xorriso, xorriso->boot_image_cat_path, 0);
