@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -243,7 +243,7 @@ int Xorriso_pvd_info(struct XorrisO *xorriso, int flag)
  IsoImage *image;
  struct burn_drive_info *dinfo;
  struct burn_drive *drive;
- char *msg, block_head[8];
+ char *msg, block_head[8], *crt, *mdt, *ext, *eft;
  off_t head_count;
 
  msg= xorriso->result_line;
@@ -292,6 +292,19 @@ int Xorriso_pvd_info(struct XorrisO *xorriso, int flag)
  Xorriso_result(xorriso,0);
  sprintf(msg, "Biblio File  : %s\n", xorriso->biblio_file);
  Xorriso_result(xorriso,0);
+
+ ret= iso_image_get_pvd_times(image, &crt, &mdt, &ext, &eft);
+ if(ret != ISO_SUCCESS)
+   crt= mdt= ext= eft= "";
+ sprintf(msg, "Creation Time: %s\n", crt);
+ Xorriso_result(xorriso,0);
+ sprintf(msg, "Modif. Time  : %s\n", mdt);
+ Xorriso_result(xorriso,0);
+ sprintf(msg, "Expir. Time  : %s\n", ext);
+ Xorriso_result(xorriso,0);
+ sprintf(msg, "Eff. Time    : %s\n", eft);
+ Xorriso_result(xorriso,0);
+
  return(1);
 }
 
@@ -378,7 +391,7 @@ int Xorriso_boot_item_status(struct XorrisO *xorriso, char *cat_path,
                      "/boot/isolinux/boot.cat") == 0)
      strcpy(bspec, "dir=/boot/isolinux");
    memset(zeros, 0, 28);
-   if(bspec[0] && platform_id == 0 && patch_isolinux &&
+   if(bspec[0] && platform_id == 0 && (patch_isolinux & 3) &&
       load_size == 2048 && is_default_id && emul == 0) {
      sprintf(line, "-boot_image isolinux %s\n", bspec);
      Xorriso_status_result(xorriso,filter,fp,flag&2); 
@@ -390,7 +403,7 @@ int Xorriso_boot_item_status(struct XorrisO *xorriso, char *cat_path,
    if(ret == 0)
      file_size= ((stbuf.st_size / (off_t) 512) +
                 !!(stbuf.st_size % (off_t) 512)) * 512;
-   if(platform_id == 0xef && !patch_isolinux &&
+   if(platform_id == 0xef && !(patch_isolinux & 3) &&
       load_size == file_size && is_default_id && emul == 0) {
      sprintf(line, "-boot_image any efi_path=");
      Text_shellsafe(bin_path, line, 1);
@@ -588,8 +601,10 @@ no_image:;
  }
  if(bin_path_in_use) {
    is_default= !xorriso->boot_image_cat_hidden;
-   hpt= Xorriso__hide_mode_text(xorriso->boot_image_cat_hidden & 3, 0);
-   sprintf(line, "-boot_image %s cat_hidden=%s\n", form, hpt);
+   hpt= Xorriso__hide_mode_text(xorriso->boot_image_cat_hidden & 63, 0);
+   if(hpt != NULL)
+     sprintf(line, "-boot_image %s cat_hidden=%s\n", form, hpt);
+   Xorriso_free_meM(hpt);
    if(!(is_default && no_defaults))
      Xorriso_status_result(xorriso,filter,fp,flag&2);
  }
@@ -998,6 +1013,10 @@ int Xorriso_mount(struct XorrisO *xorriso, char *dev, int adr_mode,
    aquire_flag= 1;
    if((xorriso->toc_emulation_flag & 2) && adr_mode == 3)
      aquire_flag|= 16;
+   if(xorriso->toc_emulation_flag & 4)
+     aquire_flag|= 128;
+   if(xorriso->toc_emulation_flag & 8)
+     aquire_flag|= 512;
    ret= isoburn_drive_aquire(&dinfo, libburn_adr, aquire_flag);
    burn_preset_device_open(1, 0, 0);
    Xorriso_process_msg_queues(xorriso,0);
