@@ -23,6 +23,7 @@
 #include <sys/time.h>
 #include <time.h>
 #include <errno.h>
+#include <pthread.h>
 
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
@@ -351,12 +352,12 @@ int Xorriso__sev_to_text(int severity, char **severity_name,
 {
  int ret;
 
-  ret= iso_sev_to_text(severity, severity_name);
-  if(ret>0)
-    return(ret);
-  ret= burn_sev_to_text(severity, severity_name, 0);
-  if(ret>0)
-    return(ret);
+ ret= iso_sev_to_text(severity, severity_name);
+ if(ret>0)
+   return(ret);
+ ret= burn_sev_to_text(severity, severity_name, 0);
+ if(ret>0)
+   return(ret);
  *severity_name= "";
  return(0);
 }
@@ -376,6 +377,31 @@ int Xorriso__text_to_sev(char *severity_name, int *severity_number, int flag)
 }
 
 
+int Xorriso__severity_cmp(char *sev1, char *sev2)
+{
+ int s1= 0x7fffffff, s2= 0x7fffffff, ret;
+ char *default_sev= "FATAL";
+
+ ret= Xorriso__text_to_sev(sev1, &s1, 0);
+ if(ret <= 0)
+   Xorriso__text_to_sev(default_sev, &s1, 0);
+ ret= Xorriso__text_to_sev(sev2, &s2, 0);
+ if(ret <= 0)
+   Xorriso__text_to_sev(default_sev, &s2, 0);
+ if(s1 < s2)
+   return -1;
+ if(s1 > s2)
+   return(1);
+ return(0);
+}
+
+
+char *Xorriso__severity_list(int flag)
+{
+ return(burn_list_sev_texts(0));
+}
+
+
 /* @param flag bit0= report libisofs error text
                bit1= victim is disk_path
                bit2= do not inquire libisofs, report msg_text and min_severity
@@ -470,25 +496,49 @@ int Xorriso_process_msg_queues(struct XorrisO *xorriso, int flag)
 {
  int ret, error_code= 0, os_errno= 0, count= 0, pass, imgid, tunneled;
  int name_prefix_code;
- char severity[80];
+ char severity[80], *text= NULL;
+
+#ifdef Xorriso_fetch_with_msg_queueS
+ int locked= 0, uret;
+#endif
 
 #ifdef Xorriso_with_libjtE
  char *msg;
 #endif
 
- if(!xorriso->libs_are_started)
-   return(1);
+ if(!xorriso->libs_are_started) {
+   ret= 1; goto ex;
+ }
+
+#ifdef Xorriso_fetch_with_msg_queueS
+
+ Xorriso_alloc_meM(text, char, sizeof(xorriso->info_text));
+
+ ret= pthread_mutex_lock(&(xorriso->lib_msg_queue_lock));
+ if(ret != 0) {
+   Xorriso_msgs_submit(xorriso, 0,
+              "Cannot aquire mutex lock for processing library message queues",
+               ret, "FATAL", 0);
+ } else
+   locked= 1;
+
+#else /* Xorriso_fetch_with_msg_queueS */
+
+ text= xorriso->info_text;
+
+#endif /* ! Xorriso_fetch_with_msg_queueS */
+
+
  for(pass= 0; pass< 3; pass++) {
    while(1) {
      tunneled= 0;
      if(pass==0) {
-
        ret= 0;
 #ifdef Xorriso_with_libjtE
        if(xorriso->libjte_handle != NULL) {
          msg= libjte_get_next_message(xorriso->libjte_handle);
          if(msg != NULL) {
-           sprintf(xorriso->info_text, "%1.4095s", msg);
+           sprintf(text, "%1.4095s", msg);
            free(msg);
            strcpy(severity, "NOTE");
            error_code= 0;
@@ -499,11 +549,9 @@ int Xorriso_process_msg_queues(struct XorrisO *xorriso, int flag)
 #endif /* Xorriso_with_libjtE */
 
      } else if(pass==1)
-       ret= iso_obtain_msgs("ALL", &error_code, &imgid,
-                            xorriso->info_text, severity);
+       ret= iso_obtain_msgs("ALL", &error_code, &imgid, text, severity);
      else {
-       ret= burn_msgs_obtain("ALL", &error_code, xorriso->info_text, &os_errno,
-                             severity);
+       ret= burn_msgs_obtain("ALL", &error_code, text, &os_errno, severity);
        if((error_code>=0x00030000 && error_code<0x00040000) ||
           (error_code>=0x00050000 && error_code<0x00060000))
          tunneled= -1; /* "libisofs:" */
@@ -526,17 +574,44 @@ int Xorriso_process_msg_queues(struct XorrisO *xorriso, int flag)
        name_prefix_code= 0;
      else
        name_prefix_code= pass + tunneled;
-     Xorriso_msgs_submit(xorriso, error_code, xorriso->info_text, os_errno, 
+     Xorriso_msgs_submit(xorriso, error_code, text, os_errno, 
                          severity, name_prefix_code << 2);
      count++;
    }
  }
  if(xorriso->library_msg_direct_print && count>0) {
-   sprintf(xorriso->info_text,"   (%d library messages repeated by xorriso)\n",
-           count);
+   sprintf(text,"   (%d library messages repeated by xorriso)\n", count);
+
+#ifdef Xorriso_fetch_with_msg_queueS
+
+   Xorriso_msgs_submit(xorriso, 0, text, 0, "NOTE", 256);
+
+#else /* Xorriso_fetch_with_msg_queueS */
+
    Xorriso_info(xorriso, 0);
+
+#endif /* Xorriso_fetch_with_msg_queueS */
+
  }
- return(1);
+ ret= 1;
+ex:;
+
+#ifdef Xorriso_fetch_with_msg_queueS
+
+ if(locked) {
+   uret= pthread_mutex_unlock(&(xorriso->lib_msg_queue_lock));
+   if(uret != 0) {
+     Xorriso_msgs_submit(xorriso, 0,
+             "Cannot release mutex lock for processing library message queues",
+              uret, "FATAL", 0);
+     ret= -1;
+   }
+ }
+ Xorriso_free_meM(text);
+
+#endif /* Xorriso_fetch_with_msg_queueS */
+
+ return(ret);
 }
 
 
