@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -172,7 +172,7 @@ int Xorriso_option_dialog(struct XorrisO *xorriso, char *mode, int flag)
  if(strcmp(mode, "on") == 0 || strcmp(mode, "multi_line") == 0)
    xorriso->dialog= 2;
  else if(strcmp(mode, "single_line") == 0)
-   xorriso->dialog= 2;
+   xorriso->dialog= 1;
  else if(strcmp(mode, "off") == 0)
    xorriso->dialog= 0;
  else {
@@ -1537,6 +1537,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 displacement versus the start address for which the image",
 "                 was prepared.",
 "  -rom_toc_scan \"on\"|\"force\"|\"off\"[:\"emul_on\"|\"emul_off\"]",
+"                [:\"emul_wide\"|\"emul_narrow\"]",
 "                 Enable scanning for ISO sessions on read-only drives/media",
 "                 resp. on overwriteable media with emulated TOC.",
 "  -calm_drive \"in\"|\"out\"|\"all\"|\"on\"|\"off\"",
@@ -1577,8 +1578,9 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Classify stdio drives by effective access permissions.",
 "  -data_cache_size number_of_tiles blocks_per_tile",
 "                 Adjust size and granularity of the data read cache.",
-"  -blank \"fast\"|\"all\"|\"deformat\"|\"deformat_quickest\"",
+"  -blank [\"force:\"]\"fast\"|\"all\"|\"deformat\"|\"deformat_quickest\"",
 "                 Blank medium resp. invalidate ISO image on medium.",
+"                 Prefix \"force:\" overrides medium evaluation.",
 "  -close_damaged \"as_needed\"|\"force\"",
 "                 Close track and session of damaged medium.",
 "  -format \"as_needed\"|\"full\"|\"fast\"|\"by_index_#\"|\"by_size_#\"",
@@ -1656,7 +1658,9 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -device_links  Like devices, but showing link paths which are hopefully",
 "                 persistent over reboot on modern Linux systems.",
 "",
-"  -toc           Show media specific table of content (sessions).",
+"  -toc           Show media specific tables of content (sessions).",
+"  -toc_of \"in\"|\"out\"|\"all\"[\":short\"]",
+"                 Show -toc of either input drive or output drive or both.",
 "",
 "  -mount_cmd drive entity id path",
 "                 Print to result channel a command suitable to mount the",
@@ -1734,8 +1738,12 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Delete the given files from the ISO image.",
 "  -rm_r iso_rr_path [***]",
 "                 Delete the given directory trees from ISO image.",
+"  -move iso_rr_path iso_rr_path",
+"                 Rename the single file given by the first iso_rr_path to",
+"                 the second iso_rr_path.",
 "  -mv iso_rr_path [***] iso_rr_path",
-"                 Rename the given file objects in the ISO tree to iso_rr_path.",
+"                 Like shell command mv rename the given file objects in the",
+"                 ISO tree to the last of the iso_rr_path parameters.",
 "  -chown uid iso_rr_path [***]",
 "                 Equivalent to chown in the ISO image.",
 "  -chown_r uid iso_rr_path [***]",
@@ -1795,6 +1803,8 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 params are their parameters except iso_rr_path.",
 "  -mkdir iso_rr_path [...]",
 "                 Create empty directories if they do not exist yet.",
+"  -lns target_text iso_rr_path",
+"                 Create a symbolic link pointing to target_text",
 "  -rmdir iso_rr_path [***]",
 "                 Delete empty directories.",
 "  -clone iso_rr_path_original iso_rr_path_copy",
@@ -1967,7 +1977,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Like -show_stream but affecting all files below directories.",
 "",
 "Restore commands which copy file objects from ISO image to disk filesystem:",
-"  -osirrox \"on\"|\"device_files\"|\"off\"|\"banned\"",
+"  -osirrox \"on\"|\"device_files\"|\"off\"|\"blocked\"|\"unblock\"|\"banned\"",
 "           [:\"concat_split_on\"|\"concat_split_off\"]",
 "           [:\"auto_chmod_on\"|\"auto_chmod_off\"]",
 "           [:\"sort_lba_on\"|\"sort_lba_off\"]",
@@ -1975,7 +1985,8 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 By default \"off\" the inverse operation of xorriso from ISO",
 "                 image to disk filesystem is disabled. \"on\" allows xorriso",
 "                 to create, overwrite, delete files in the disk filesystem.", 
-"                 \"banned\" is irrevocably \"off\".",
+"                 \"banned\" is irrevocably \"off\". \"blocked\" can only be",
+"                 revoked by \"unblock\". (\"device_files\" is dangerous.)",
 "  -extract iso_rr_path disk_path",
 "                 Copy tree under iso_rr_path onto disk address disk_path.",
 "                 This avoids the pitfalls of cp -r addressing rules.",
@@ -2072,6 +2083,14 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "              packet payload. Example of a info message with newline:",
 "                I:1: enter option text :",
 "              -pkt_output:on is intended for use by frontend programs.",
+"  -msg_op \"start_sieve\"|\"read_sieve\"|\"clear_sieve\"|\"end_sieve\"|",
+"          \"parse\"|\"parse_bulk\"|\"compare_sev\"|\"list_sev\" param_text",
+"              Enable, use, or disable message sieve. Or parse lines into",
+"              words. Or compare or list severity names.",
+"  -launch_frontend program [args ...] --",
+"              Start a program, connect its stdin to xorriso stdout and",
+"              stderr, connect its stdout to xorriso stdin.",
+"              Use any given parameters as arguments for the started program.",
 "  -logfile channel fileaddress  Copy output of a channel to the given file.",
 "              channel may be 'R','I','M' as with -pkt_output or '.'",
 "              for the consolidated -pkt_output stream.",
