@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -74,6 +74,34 @@ int Xorriso_option_joliet(struct XorrisO *xorriso, char *mode, int flag)
 }
 
 
+/* Command -launch_frontend */
+int Xorriso_option_launch_frontend(struct XorrisO *xorriso,
+                                   int argc, char **argv, int *idx, int flag)
+{
+ int ret, end_idx;
+
+ end_idx= Xorriso_end_idx(xorriso, argc, argv, *idx, 1);
+
+ if(xorriso->launch_frontend_banned) {
+   sprintf(xorriso->info_text,
+           "-launch_frontend was already executed in this xorriso run");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
+   ret= 0; goto ex;
+ }
+ xorriso->launch_frontend_banned= 1;
+ if(end_idx <= *idx)
+   {ret= 1; goto ex;}
+ if(argv[*idx][0] == 0)
+   {ret= 1; goto ex;}
+ xorriso->dialog= 2;
+ ret= Xorriso_launch_frontend(xorriso, end_idx - *idx, argv + *idx,
+                              "", "", 0);
+ex:;
+ (*idx)= end_idx;
+ return(ret);
+}
+
+
 /* Option -list_arg_sorting */
 int Xorriso_option_list_arg_sorting(struct XorrisO *xorriso, int flag)
 {
@@ -182,6 +210,38 @@ int Xorriso_option_list_profiles(struct XorrisO *xorriso, char *which,
 }
 
 
+/* Command -lns alias -lnsi */
+int Xorriso_option_lnsi(struct XorrisO *xorriso, char *target, char *path,
+                        int flag)
+{
+ int ret;
+ char *eff_path= NULL;
+
+ Xorriso_alloc_meM(eff_path, char, SfileadrL);
+
+ ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, path, eff_path, 1);
+ if(ret < 0)
+   {ret= 0; goto ex;}
+ if(ret > 0) { 
+   sprintf(xorriso->info_text, "-lns: Address already existing: ");
+   Text_shellsafe(eff_path, xorriso->info_text, 1);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   {ret= 0; goto ex;}
+ } 
+ ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, path, eff_path, 2);
+ if(ret < 0)
+   {ret= 0; goto ex;}
+ ret= Xorriso_graft_in(xorriso, NULL, target, eff_path, (off_t) 0, (off_t) 0,
+                       1024);
+ if(ret <= 0)
+   {ret= 0; goto ex;}
+ ret= 1;
+ex:;
+ Xorriso_free_meM(eff_path);
+ return(ret);
+}
+
+
 /* Option -load session|track|sbsector value */
 /* @param flag bit0= with adr_mode sbsector: adr_value is possibly 16 too high
    @return <=0 error , 1 success, 2 revoked by -reassure
@@ -806,6 +866,155 @@ unknown_mode:;
 }
 
 
+/* Command -move */
+int Xorriso_option_move(struct XorrisO *xorriso, char *origin, char *dest,
+                        int flag)
+{
+ int ret;
+ char *eff_origin= NULL, *eff_dest= NULL;
+
+ Xorriso_alloc_meM(eff_origin, char, SfileadrL);
+ Xorriso_alloc_meM(eff_dest, char, SfileadrL);
+
+ ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, origin, eff_origin, 0);
+ if(ret <= 0)
+   goto ex;
+ ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, dest, eff_dest, 0);
+ if(ret <= 0)
+   goto ex;
+ ret= Xorriso_rename(xorriso, NULL, eff_origin, eff_dest, 0);
+ if(ret <= 0)
+   goto ex;
+ ret= 1;
+ex:;
+ Xorriso_free_meM(eff_origin);
+ Xorriso_free_meM(eff_dest);
+ return(ret);
+}
+
+ 
+/* Command -msg_op */
+int Xorriso_option_msg_op(struct XorrisO *xorriso, char *what, char *arg,
+                          int flag)
+{
+ int ret, available, argc, pargc, i, pflag, max_words, input_lines, msd_mem;
+ char **argv= NULL, **pargv= NULL, *msg= "";
+ char *prefix, *separators;
+
+ msd_mem= xorriso->msg_sieve_disabled;
+
+ ret= 1;
+ if(strcmp(what, "parse") == 0 || strcmp(what, "parse_bulk") == 0) {
+   ret= Xorriso_parse_line(xorriso, arg, "", "", 5, &argc, &argv, 0);
+   prefix= "";
+   if(argc > 0) 
+     prefix= argv[0];
+   separators= "";
+   if(argc > 1)
+     separators= argv[1];
+   max_words= 0;
+   if(argc > 2)
+     sscanf(argv[2], "%d", &max_words);
+   pflag= 0;
+   if(argc > 3)
+     sscanf(argv[3], "%d", &pflag);
+   input_lines= 1;
+   if(argc > 4)
+     sscanf(argv[4], "%d", &input_lines);
+   if(strcmp(what, "parse") == 0) {
+     ret= Xorriso_msg_op_parse(xorriso, "", prefix, separators,
+                               max_words, pflag, input_lines, 0);
+   } else {
+     ret= Xorriso_msg_op_parse_bulk(xorriso, prefix, separators,
+                               max_words, pflag, input_lines, 0);
+   }
+   if(ret <= 0)
+     goto ex;
+   xorriso->msg_sieve_disabled= msd_mem;
+   Xorriso__dispose_words(&argc, &argv);
+ } else if(strcmp(what, "start_sieve") == 0) {
+   Xorriso_sieve_dispose(xorriso, 0);
+   ret= Xorriso_sieve_big(xorriso, 0);
+   msg= "Message sieve enabled";
+
+/* >>> } else if(strcmp(what, "add_filter_rule") == 0) { */;
+
+ } else if(strcmp(what, "clear_sieve") == 0) {
+   ret= Xorriso_sieve_clear_results(xorriso, 0);
+   msg= "Recorded message sieve results disposed";
+ } else if(strcmp(what, "end_sieve") == 0) {
+   ret= Xorriso_sieve_dispose(xorriso, 0);
+   msg= "Message sieve disabled";
+ } else if(strcmp(what, "read_sieve") == 0) {
+   ret= Xorriso_sieve_get_result(xorriso, arg, &pargc, &pargv, &available, 0);
+   xorriso->msg_sieve_disabled= 1;
+   sprintf(xorriso->result_line, "%d\n", ret);
+   Xorriso_result(xorriso, 1);
+   if(ret > 0) {
+     sprintf(xorriso->result_line, "%d\n", pargc);
+     Xorriso_result(xorriso, 1);
+     for(i= 0; i < pargc; i++) {
+       ret= Sfile_count_char(pargv[i], '\n') + 1;
+       sprintf(xorriso->result_line, "%d\n", ret);
+       Xorriso_result(xorriso, 1);
+       Sfile_str(xorriso->result_line, pargv[i], 0);
+       strcat(xorriso->result_line, "\n");
+       Xorriso_result(xorriso, 1);
+     }
+   } else {
+     strcpy(xorriso->result_line, "0\n");
+     Xorriso_result(xorriso, 1);
+     available= 0;
+   }
+   sprintf(xorriso->result_line, "%d\n", available);
+   Xorriso_result(xorriso, 1);
+   xorriso->msg_sieve_disabled= msd_mem;
+   Xorriso__dispose_words(&pargc, &pargv);
+   ret= 1;
+ } else if(strcmp(what, "show_sieve") == 0) {
+   ret= Xorriso_sieve_get_result(xorriso, "", &pargc, &pargv, &available, 8);
+   xorriso->msg_sieve_disabled= 1;
+   sprintf(xorriso->result_line, "%d\n", ret);
+   Xorriso_result(xorriso, 1);
+   if(ret > 0) {
+     sprintf(xorriso->result_line, "%d\n", pargc);
+     Xorriso_result(xorriso, 1);
+     for(i= 0; i < pargc; i++) {
+       sprintf(xorriso->result_line, "%s\n", pargv[i]);
+       Xorriso_result(xorriso, 1);
+     }
+   }
+   xorriso->msg_sieve_disabled= msd_mem;
+   Xorriso__dispose_words(&pargc, &pargv);
+ } else if(strcmp(what, "compare_sev") == 0) {
+   ret= Xorriso_parse_line(xorriso, arg, "", ",", 2, &argc, &argv, 0);
+   if(argc < 2) {
+     sprintf(xorriso->info_text,
+             "-msg_op cmp_sev: malformed severity pair '%s'", arg);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
+   } else {
+     ret= Xorriso__severity_cmp(argv[0], argv[1]);
+     sprintf(xorriso->result_line, "%d\n", ret);
+     Xorriso_result(xorriso, 1);
+   }
+   Xorriso__dispose_words(&argc, &argv);
+ } else if(strcmp(what, "list_sev") == 0) {
+   sprintf(xorriso->result_line, "%s\n", Xorriso__severity_list(0));
+   Xorriso_result(xorriso, 1);
+ } else {
+   sprintf(xorriso->info_text, "-msg_op: unknown operation '%s'", what);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
+   ret= 0;
+ }
+ if(ret > 0 && msg[0])
+   Xorriso_msgs_submit(xorriso, 0, msg, 0, "NOTE", 0);
+
+ex:;
+ xorriso->msg_sieve_disabled= msd_mem;
+ return(ret);
+}
+
+
 /* Option -mv alias -mvi */
 int Xorriso_option_mvi(struct XorrisO *xorriso, int argc, char **argv,
                       int *idx, int flag)
@@ -1283,7 +1492,13 @@ int Xorriso_option_osirrox(struct XorrisO *xorriso, char *mode, int flag)
      allow_restore= 0;
    else if(strncmp(cpt, "banned", l)==0 && l >= 5)
      allow_restore= -1;
-   else if(strncmp(cpt, "device_files", l)==0 && l >= 12)
+   else if(strncmp(cpt, "blocked", l)==0 && l >= 7)
+     allow_restore= -2;
+   else if(strncmp(cpt, "unblock", l)==0 && l >= 7) {
+     if(xorriso->allow_restore == -2)
+       xorriso->allow_restore= 0;
+     allow_restore= 1;
+   } else if(strncmp(cpt, "device_files", l)==0 && l >= 12)
      allow_restore= 2;
    else if((strncmp(cpt, "on", l)==0 && l >= 2) || mode[0]==0)
      allow_restore= 1;
@@ -1320,6 +1535,12 @@ unknown_mode:;
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
    return(0);
  }
+ if(allow_restore > 0 && xorriso->allow_restore == -2) {
+   sprintf(xorriso->info_text,
+           "-osirrox: is currently disabled by setting 'blocked'");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ }
  if(xorriso->allow_restore != -1)
    xorriso->allow_restore= allow_restore;
  sprintf(xorriso->info_text,
