@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -30,12 +30,16 @@
 #include <sys/wait.h>
 
 
-
 #include "xorriso.h"
 #include "xorriso_private.h"
 #include "xorrisoburn.h"
 
 
+#ifdef Xorriso_fetch_with_msg_queueS
+#include <pthread.h>
+#endif
+
+
 /* @param flag bit0= do not warn of wildcards
                bit1= these are disk_paths
 */
@@ -207,17 +211,55 @@ int Xorriso_set_problem_status(struct XorrisO *xorriso, char *severity,
  char *sev_text= "ALL";
  int sev, ret;
 
+#ifdef Xorriso_fetch_with_msg_queueS
+ int locked= 0, uret;
+ static int complaints= 0, complaint_limit= 5;
+#endif
+
  if(severity[0])
    sev_text= severity;
  ret= Xorriso__text_to_sev(sev_text, &sev, 0);
  if(ret<=0)
    return(0);
+
+#ifdef Xorriso_fetch_with_msg_queueS
+
+ ret= pthread_mutex_lock(&(xorriso->problem_status_lock));
+ if(ret != 0) {
+   /* Cannot report failure through the failing message output system */
+   complaints++;
+   if(complaints < complaint_limit)
+     fprintf(stderr,
+        "xorriso : pthread_mutex_lock() for problem_status returns %d\n",
+        ret);
+ } else
+   locked= 1;
+
+#endif /* Xorriso_fetch_with_msg_queueS */
+
  xorriso->problem_status= sev;
  strcpy(xorriso->problem_status_text, sev_text);
  if(sev > xorriso->eternal_problem_status) {
    xorriso->eternal_problem_status= sev;
    strcpy(xorriso->eternal_problem_status_text, sev_text);
  }
+
+#ifdef Xorriso_fetch_with_msg_queueS
+
+ if(locked) {
+   uret= pthread_mutex_unlock(&(xorriso->problem_status_lock));
+   if(uret != 0) {
+     /* Cannot report failure through the failing message output system */
+     complaints++;
+     if(complaints < complaint_limit)
+       fprintf(stderr,
+            "xorriso : pthread_mutex_unlock() for problem_status returns %d\n",
+            uret);
+   }
+ }
+
+#endif /* Xorriso_fetch_with_msg_queueS */
+
  return(1);
 }
 
@@ -292,12 +334,12 @@ int Xorriso_cpmv_args(struct XorrisO *xorriso, char *cmd,
  }
 
  ret= Xorriso_opt_args(xorriso, cmd, argc, argv, *idx, &end_idx, optc, optv,
-                       (flag&2)|16); /* ignore last argument */
+                       1 | (flag&2) | 16); /* ignore last argument */
  if(ret<=0)
    goto ex;
  /* demand one match, or 0 with a constant */
  ret= Xorriso_opt_args(xorriso, cmd, argc, argv, end_idx, &end_idx, &destc,
-                       &destv, ((flag&4)>>1) | 32 | 64);
+                       &destv, 1 | ((flag&4)>>1) | 32 | 64);
  if(ret<=0)
    goto ex;
 
@@ -477,7 +519,7 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "rom_toc_scan","rr_reloc_dir","scsi_log",
     "session_log","signal_handling","sleep",
     "speed","split_size","status","status_history_max",
-    "stdio_sync","stream_recording","system_id","temp_mem_limit",
+    "stdio_sync","stream_recording","system_id","temp_mem_limit","toc_of",
     "uid","unregister_filter","use_readline","volid","volset_id",
     "write_type","xattr","zisofs",
     ""
@@ -486,8 +528,8 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "assert_volid","boot_image","clone","compare","compare_r","drive_class",
     "data_cache_size",
     "errfile_log","error_behavior","extract","extract_single",
-    "jigdo","load","logfile",
-    "map","map_single","page","return_with",
+    "jigdo","lns","lnsi","load","logfile",
+    "map","map_single","move","msg_op","page","return_with",
     "scdbackup_tag","update","update_r","volume_date",
     ""
  };
@@ -509,7 +551,7 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "file_size_limit","find","findi","finds","findx",
     "getfacl","getfacli","getfacl_r","getfacl_ri",
     "getfattr","getfattri","getfattr_r","getfattr_ri","hide",
-    "ls","lsi","lsl","lsli","lsd","lsdi","lsdl","lsdli",
+    "launch_frontend","ls","lsi","lsl","lsli","lsd","lsdi","lsdl","lsdli",
     "lsx","lslx","lsdx","lsdlx","map_l","mv","mvi","mkdir","mkdiri",
     "not_paths","rm","rmi","rm_r","rm_ri","rmdir","rmdiri","update_l",
     "setfacl","setfacli","setfacl_list","setfacl_listi",
@@ -618,7 +660,7 @@ int Xorriso_cmd_sorting_rank(struct XorrisO *xorriso,
 
    "* Drive and media related inquiry actions (2):",
    "list_profiles", "list_formats", "list_speeds",
-   "toc", "pvd_info",
+   "toc", "toc_of", "pvd_info",
 
    "* Settings for file insertion:",
    "file_size_limit", "not_mgt", "not_paths", "not_leaf", "not_list",
@@ -629,7 +671,7 @@ int Xorriso_cmd_sorting_rank(struct XorrisO *xorriso,
 
    "* Inserting files into ISO image:",
    "disk_pattern", "add_plainly",
-   "mkdir", "add", "path_list", "quoted_path_list",
+   "mkdir", "lns", "add", "path_list", "quoted_path_list",
    "map", "map_single", "map_l", "update", "update_r", "update_l",
    "cut_out", "cpr", 
    "clone", "cp_clone",
@@ -642,7 +684,7 @@ int Xorriso_cmd_sorting_rank(struct XorrisO *xorriso,
 
    "* File manipulations:",
    "iso_rr_pattern",
-   "rm", "rm_r", "rmdir", "mv",
+   "rm", "rm_r", "rmdir", "move", "mv",
    "chown", "chown_r", "chgrp", "chgrp_r", "chmod", "chmod_r", "setfacl",
    "setfacl_r", "setfacl_list", "setfattr", "setfattr_r", "setfattr_list",
    "alter_date", "alter_date_r", "hide",
@@ -693,11 +735,14 @@ int Xorriso_cmd_sorting_rank(struct XorrisO *xorriso,
    "check_media_defaults", "check_media", "check_md5", "check_md5_r",
 
    "* Support for frontend programs via stdin and stdout (2):",
-   "pkt_output", "logfile", "mark",
+   "pkt_output", "logfile", "mark", "msg_op",
 
    "* Dialog mode control:",
    "dialog", "page", "use_readline", "reassure",
 
+   "* Support for frontend programs via stdin and stdout (3):",
+   "launch_frontend",
+
    "* Scripting, dialog and program control features (3):",
    "rollback_end", "end",
 
@@ -857,7 +902,7 @@ return:
 {
  int ret, was_dashed, end_ret, num1, num2, cmd_data_size= 2 * SfileadrL;
  int mem_idx, arg_count, i;
- char *cmd, *original_cmd, *cmd_data= NULL, *arg1, *arg2;
+ char *cmd, *original_cmd, *cmd_data= NULL, *arg1, *arg2, *arg3;
  
  Xorriso_alloc_meM(cmd_data, char, cmd_data_size);
 
@@ -932,6 +977,10 @@ next_command:;
    arg2= argv[(*idx)+1];
  else
    arg2= "";
+ if((*idx) + 2 < argc)
+   arg3= argv[(*idx) + 2];
+ else
+   arg3= "";
 
  ret= 1;
  if(cmd[0]=='#' || cmd[0]==0) {
@@ -963,8 +1012,7 @@ next_command:;
 
  } else if(strcmp(cmd,"append_partition")==0) {
    (*idx)+= 3;
-   ret= Xorriso_option_append_partition(xorriso, arg1, arg2, argv[*idx - 1],
-                                        0);
+   ret= Xorriso_option_append_partition(xorriso, arg1, arg2, arg3, 0);
 
  } else if(strcmp(cmd,"application_id")==0) {
    (*idx)++;
@@ -1308,6 +1356,9 @@ next_command:;
    (*idx)++;
    ret= Xorriso_option_joliet(xorriso, arg1, 0);
 
+ } else if(strcmp(cmd, "launch_frontend") == 0) {
+   ret= Xorriso_option_launch_frontend(xorriso, argc, argv, idx, 0);
+
  } else if(strcmp(cmd, "list_arg_sorting") == 0) {
    ret= Xorriso_option_list_arg_sorting(xorriso, 0);
 
@@ -1329,6 +1380,10 @@ next_command:;
  } else if(strcmp(cmd,"list_speeds")==0) {
    ret= Xorriso_option_list_speeds(xorriso, 0);
 
+ } else if(strcmp(cmd, "lns") == 0 || strcmp(cmd, "lnsi") == 0) {
+   (*idx)+= 2;
+   ret= Xorriso_option_lnsi(xorriso, arg1, arg2, 0);
+
  } else if(strcmp(cmd,"load")==0) {
    (*idx)+= 2;
    ret= Xorriso_option_load(xorriso, arg1, arg2, 0);
@@ -1391,6 +1446,14 @@ next_command:;
    (*idx)++;
    ret= Xorriso_option_mount_opts(xorriso, arg1, 0);
 
+ } else if(strcmp(cmd, "move")==0) {
+   (*idx)+= 2;
+   ret= Xorriso_option_move(xorriso, arg1, arg2, 0);
+
+ } else if(strcmp(cmd,"msg_op") == 0) {
+   (*idx)+= 2;
+   ret= Xorriso_option_msg_op(xorriso, arg1, arg2, 0);
+
  } else if(strcmp(cmd,"mv")==0 || strcmp(cmd,"mvi")==0) {
    ret= Xorriso_option_mvi(xorriso, argc, argv, idx, 0);
 
@@ -1682,9 +1745,90 @@ next_command:;
  } else if(strcmp(cmd,"test")==0) { /* This option does not exist. */
    /* install temporary test code here */;
 
+if (0) {
+/* Test setup for for Xorriso_push_outlists() et.al. */
+/* Test setup for Xorriso_parse_line */
+   int stack_handle = -1, line_count= 0;
+   struct Xorriso_lsT *result_list, *info_list;
+   int Xorriso_process_msg_lists(struct XorrisO *xorriso,
+                                     struct Xorriso_lsT *result_list,
+                                     struct Xorriso_lsT *info_list,
+                                     int *line_count, int flag);
+
+   (*idx)++;
+   if(strcmp(arg1, "push") == 0) {
+     ret= Xorriso_push_outlists(xorriso, &stack_handle, 3);
+     fprintf(stderr, "xorriso -test: push = %d, handle = %d\n",
+             ret, stack_handle);
+   } else if(strcmp(arg1, "pull") == 0) {
+     ret= Xorriso_pull_outlists(xorriso, -1, &result_list, &info_list, 0);
+     fprintf(stderr, "xorriso -test: pull = %d\n", ret);
+     if(ret > 0) {
+       ret= Xorriso_process_msg_lists(xorriso, result_list, info_list,
+                                      &line_count, 0);
+       fprintf(stderr,
+          "xorriso -test: Xorriso_process_msg_lists() = %d, line_count = %d\n",
+          ret, line_count);
+     }
+   } else if(strcmp(arg1, "fetch") == 0) {
+     ret= Xorriso_fetch_outlists(xorriso, -1, &result_list, &info_list, 0);
+     fprintf(stderr, "xorriso -test: fetch = %d\n", ret);
+     if(ret > 0) {
+       ret= Xorriso_process_msg_lists(xorriso, result_list, info_list,
+                                      &line_count, 0);
+       fprintf(stderr,
+          "xorriso -test: Xorriso_process_msg_lists() = %d, line_count = %d\n",
+          ret, line_count);
+     }
+   } else if(strcmp(arg1, "peek") == 0) {
+     ret= Xorriso_peek_outlists(xorriso, -1, 0, 0);
+     fprintf(stderr, "xorriso -test: peek = %d\n", ret);
+   } else if(strcmp(arg1, "sleep_peek") == 0) {
+     usleep(1000000);
+     ret= Xorriso_peek_outlists(xorriso, -1, 0, 0);
+     fprintf(stderr, "xorriso -test: sleep_peek = %d\n", ret);
+   } else if(strcmp(arg1, "peek_loop") == 0) {
+     ret= Xorriso_peek_outlists(xorriso, -1, 3, 4);
+     fprintf(stderr, "xorriso -test: peek_loop = %d\n", ret);
+   } else if(strcmp(arg1, "start") == 0) {
+     ret= Xorriso_start_msg_watcher(xorriso, NULL, NULL, NULL, NULL, 0);
+     fprintf(stderr, "xorriso -test: Xorriso_start_msg_watcher() = %d\n", ret);
+   } else if(strcmp(arg1, "stop") == 0) {
+     ret= Xorriso_stop_msg_watcher(xorriso, 0);
+     fprintf(stderr, "xorriso -test: Xorriso_stop_msg_watcher() = %d\n", ret);
+
+   } else if(strcmp(arg1, "help") == 0) {
+     fprintf(stderr, "-test [mode] [arguments]\n");
+     fprintf(stderr, "   push\n");
+     fprintf(stderr, "     perform Xorriso_push_outlists()\n");
+     fprintf(stderr, "   pull\n");
+     fprintf(stderr, "     perform Xorriso_pull_outlists() and show messages\n");
+     fprintf(stderr, "   fetch\n");
+     fprintf(stderr, "     perform Xorriso_fetch_outlists() and show\n");
+     fprintf(stderr, "   peek\n");
+     fprintf(stderr, "     perform Xorriso_peek_outlists()\n");
+     fprintf(stderr, "   sleep_peek\n");
+     fprintf(stderr, "     sleep 1 s and perform Xorriso_peek_outlists()\n");
+     fprintf(stderr, "   peek_loop\n");
+     fprintf(stderr, "     wait for up to 3s in Xorriso_peek_outlists()\n");
+     fprintf(stderr, "     for return value 0 or -1\n");
+     fprintf(stderr, "   start\n");
+     fprintf(stderr, "     perform Xorriso_start_msg_watcher()\n");
+     fprintf(stderr, "   stop\n");
+     fprintf(stderr, "     perform Xorriso_stop_msg_watcher()\n");
+   } else {
+     fprintf(stderr, "xorriso -test: unknwon mode: %s\n", arg1);
+   }
+   ret= 0;
+}
+
  } else if(strcmp(cmd,"toc")==0) {
    Xorriso_option_toc(xorriso, 0);
 
+ } else if(strcmp(cmd,"toc_of")==0) {
+   (*idx)++;
+   Xorriso_option_toc_of(xorriso, arg1, 0);
+
  } else if(strcmp(cmd,"uid")==0) {
    (*idx)++;
    ret= Xorriso_option_uid(xorriso,arg1,0);
@@ -1745,7 +1889,7 @@ next_command:;
    if(xorriso->add_plainly>1)
      goto add_plain_argument;
 unknown_option:;
-   sprintf(xorriso->info_text, "Not a known option:  '%s'\n",
+   sprintf(xorriso->info_text, "Not a known command:  '%s'\n",
            original_cmd);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
    {ret= 0; goto eval_any_problems;}
@@ -1776,6 +1920,66 @@ ex:;
 }
 
 
+int Xorriso_parse_line(struct XorrisO *xorriso, char *line,
+                       char *prefix, char *separators, int max_words,
+                       int *argc, char ***argv, int flag)
+{
+ int ret, bsl_mode;
+ char *to_parse, *progname= "";
+
+ if(xorriso == NULL && (flag & (32 | 64))) {
+   ret= -2; goto ex;
+ }
+
+ *argc= 0;
+ *argv= NULL;
+
+ to_parse= line;
+ if((flag & 1) || xorriso == NULL)
+   bsl_mode= (flag >> 1) & 3;
+ else
+   bsl_mode= xorriso->bsl_interpretation & 3;
+ if(prefix[0]) {
+   if(strncmp(line, prefix, strlen(prefix)) == 0) {
+     to_parse= line + strlen(prefix);
+   } else {
+     ret= 2; goto ex;
+   }
+ }
+
+ if(xorriso != NULL)
+   progname= xorriso->progname;
+ ret= Sfile_sep_make_argv(progname, to_parse, separators,
+                          max_words, argc, argv,
+                          (!(flag & 32)) | 4 | (bsl_mode << 5));
+ if(ret < 0) {
+   if(xorriso != NULL)
+     Xorriso_msgs_submit(xorriso, 0,
+        "Severe lack of resources during command line parsing", 0, "FATAL", 0);
+   ret= -1; goto ex;
+ }
+ if(ret == 0) {
+   if((flag & 64) && xorriso != NULL) {
+     sprintf(xorriso->info_text, "Incomplete quotation in %s line: %s",
+             (flag & 32) ? "command" : "parsed", to_parse);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   }
+   goto ex;
+ }
+ ret= 1;
+ex:;
+ if(ret <= 0)
+   Sfile_sep_make_argv("", "", "", 0, argc, argv, 2); /* release memory */
+ return(ret);
+}
+
+
+void Xorriso__dispose_words(int *argc, char ***argv)
+{
+ Sfile_make_argv("", "", argc, argv, 2); /* release memory */
+}
+
+
 int Xorriso_execute_option(struct XorrisO *xorriso, char *line, int flag)
 /*
  bit0-bit15 are forwarded to Xorriso_interpreter
@@ -1795,20 +1999,10 @@ int Xorriso_execute_option(struct XorrisO *xorriso, char *line, int flag)
  xorriso->idle_time= 0.0;
  tdiff= tv.tv_sec+(1.e-6*(double) tv.tv_usec);
 
- /* parse line into args */
- ret= Sfile_make_argv(xorriso->progname, line, &argc, &argv,
-                      4 | ((xorriso->bsl_interpretation & 3) << 5));
- if(ret < 0) {
-   Xorriso_msgs_submit(xorriso, 0,
-      "Severe lack of resources during command line parsing", 0, "FATAL", 0);
+ ret= Xorriso_parse_line(xorriso, line, "", "", 0, &argc, &argv, 32 | 64);
+ if(ret <= 0)
    goto ex;
- }
- if(ret == 0) {
-   sprintf(xorriso->info_text, "Incomplete quotation in command line: %s",
-           line);
-   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
-   goto ex;
- }
+
  if(argc<2)
    {ret= 1; goto ex;}
  if(argv[1][0]=='#')
@@ -1931,7 +2125,7 @@ int Xorriso_prescan_args(struct XorrisO *xorriso, int argc, char **argv,
      arg1= argv[i+1];
    arg2= "";
    if(i+2<argc)
-     arg2= argv[2+1];
+     arg2= argv[i+2];
    if(i>1)
      xorriso->did_something_useful= 1;
    if(i==1 && argc==2) {
@@ -2067,7 +2261,7 @@ protect_stdout:;
        i+= arg_count;
      } else if((flag & 2) && ((was_dashed && xorriso->add_plainly <= 1) ||
                               xorriso->add_plainly <= 0)) {
-       sprintf(xorriso->info_text, "Not a known option:  '%s'\n",
+       sprintf(xorriso->info_text, "Not a known command:  '%s'\n",
                original_cmd);
        Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
        error_seen= 1;
