@@ -2,7 +2,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -576,6 +576,12 @@ int Xorriso_attach_boot_image(struct XorrisO *xorriso, int flag)
    {ret= 0; goto ex;}
  }
  el_torito_set_boot_platform_id(bootimg, (uint8_t) platform_id);
+ if(load_size / 512 > 65535) {
+   sprintf(xorriso->info_text,
+ "Boot image load size exceeds 65535 blocks. Will record 65535 in El Torito.");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "WARNING", 0);
+   load_size= 65535 * 512;
+ }
  el_torito_set_load_size(bootimg, load_size / 512); 
  el_torito_set_id_string(bootimg, xorriso->boot_id_string);
  el_torito_set_selection_crit(bootimg, xorriso->boot_selection_crit);
@@ -588,6 +594,7 @@ int Xorriso_attach_boot_image(struct XorrisO *xorriso, int flag)
    xorriso->boot_image_bin_path[0]= 0;
    xorriso->boot_image_bin_form[0]= 0;
    xorriso->boot_image_emul= 0;
+   xorriso->boot_emul_default= 1;
    xorriso->boot_image_load_size= 4 * 512;
    memset(xorriso->boot_id_string, 0, sizeof(xorriso->boot_id_string));
    memset(xorriso->boot_selection_crit, 0,
@@ -2077,8 +2084,10 @@ int Xorriso_relax_compliance(struct XorrisO *xorriso, char *mode,
    } else if((l == 16 && strncmp(cpt, "allow_dir_id_ext", l) == 0) ||
              (l == 19 && strncmp(cpt, "allow_dir_id_ext_on", l) == 0) ) {
      xorriso->relax_compliance|= isoburn_igopt_allow_dir_id_ext;
+     xorriso->allow_dir_id_ext_dflt= 0;
    } else if((l == 20 && strncmp(cpt, "allow_dir_id_ext_off", l) == 0)) {
      xorriso->relax_compliance&= ~isoburn_igopt_allow_dir_id_ext;
+     xorriso->allow_dir_id_ext_dflt= 0;
 
    } else if((l == 12 && strncmp(cpt, "omit_version", l) == 0) ||
              (l == 15 && strncmp(cpt, "omit_version_on", l) == 0) ) {
@@ -2207,10 +2216,12 @@ int Xorriso_relax_compliance(struct XorrisO *xorriso, char *mode,
      if(value == 1 || value == 2) {
        limit= ((off_t) 4) * ((off_t) 1024*1024*1024) - ((off_t) 1);
        xorriso->iso_level= value;
+       xorriso->iso_level_is_default= 0;
        if(xorriso->file_size_limit > limit)
          xorriso->file_size_limit= limit;
      } else if(value == 3) {
        xorriso->iso_level= value;
+       xorriso->iso_level_is_default= 0;
        if(xorriso->file_size_limit < Xorriso_default_file_size_limiT)
          xorriso->file_size_limit= Xorriso_default_file_size_limiT;
      } else {
@@ -2425,7 +2436,7 @@ int Xorriso_update_iso_lba0(struct XorrisO *xorriso, int iso_lba, int isosize,
  int ret, full_size, i;
  char *headpt;
  struct burn_drive_info *dinfo;
- struct burn_drive *drive;
+ struct burn_drive *drive = NULL;
  off_t seek_ret, to_write;
  int tag_type;
  uint32_t pos, range_start, range_size, next_tag;
@@ -2455,8 +2466,9 @@ int Xorriso_update_iso_lba0(struct XorrisO *xorriso, int iso_lba, int isosize,
 
  if(!(flag & 2)) {
    /* head_buffer was not filled yet. Read it from output media. */
-   if(burn_drive_get_drive_role(drive) == 5) /* write-only */
-     return(2);
+   if(drive != NULL)
+     if(burn_drive_get_drive_role(drive) == 5) /* write-only */
+       return(2);
    if(job != NULL && job->data_to_fd >= 0) {
      if((flag & 8) && job->sector_map != NULL) {
        ret= Sectorbitmap_bytes_are_set(job->sector_map,
@@ -2484,16 +2496,18 @@ int Xorriso_update_iso_lba0(struct XorrisO *xorriso, int iso_lba, int isosize,
        Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, errno, "FAILURE",0);
        return(0);
      }
-     ret= isoburn_read_iso_head(drive, 0, &isosize, head_buffer, 1 << 13);
+     ret= isoburn_read_iso_head(NULL, 0, &isosize, head_buffer, 1 << 13);
      if(ret<=0) {
        Xorriso_process_msg_queues(xorriso,0);
        sprintf(xorriso->info_text,
-               "Alleged session start does not like ISO 9660.");
+               "Alleged session start does not look like ISO 9660.");
        Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, errno, "FAILURE",0);
        return(0);
      }
    } else {
-     ret= isoburn_read_iso_head(drive, iso_lba, &isosize, head_buffer, 2);
+     ret= 0;
+     if(drive != NULL)
+       ret= isoburn_read_iso_head(drive, iso_lba, &isosize, head_buffer, 2);
      if(ret<=0) {
        Xorriso_process_msg_queues(xorriso,0);
        sprintf(xorriso->info_text,
