@@ -2,7 +2,7 @@
 /* xorriso - libisoburn higher level API which creates, loads, manipulates
              and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -27,6 +27,13 @@
      @since major.minor.micro
    If this mark is missing, then the feature was present before release 0.5.8.
 
+   Please note that struct XorrisO and its API calls are _not_ thread-safe in
+   general. It is not permissible to run two API calls on the same
+   XorrisO object concurrently.
+   The only exception is
+     Xorriso_fetch_outlists()
+   in order to learn about the ongoing text output of other API calls.
+
 
    There is a lower level of API which consists of libisofs.h, libburn.h and
    libisoburn.h. One should not mix those calls with the ones of xorriso.h .
@@ -61,7 +68,7 @@ struct XorrisO;
 */
 #define Xorriso_header_version_majoR  1
 #define Xorriso_header_version_minoR  2
-#define Xorriso_header_version_micrO  4
+#define Xorriso_header_version_micrO  8
 
 
 /** If needed: Something like ".pl01" to indicate a bug fix. Normally empty.
@@ -287,10 +294,7 @@ int Xorriso_read_rc(struct XorrisO *xorriso, int flag);
                   (*argv)[1] to (*argv)[argc-1] contain commands and parameters
                   If argv after the call differs from argv before the call,
                   then one should dispose it later by:
-                    for(i= 0; i < argc; i++)
-                      if(argv[i] != NULL)
-                        free(argv[i]);
-                    free(argv);
+                    Xorriso__dispose_words(argc, argv);
    @param flag    unused yet, submit 0
    @return        <= 0 error , > 0 success
 */
@@ -305,7 +309,7 @@ int Xorriso_program_arg_bsl(struct XorrisO *xorriso, int argc, char ***argv,
    parameters. All parameters must be given in the same call as their command.
    @since 1.2.2:
    Commands may get arranged in a sequence that is most likely to make sense.
-   E.g. image loading settings before drive aquiration, then commands fori
+   E.g. image loading settings before drive aquiration, then commands for
    adding files, then settings for writing, then writing.
    This feature may be enabled by command "-x" in Xorriso_prescan_args()
    or by parameter flag of this call.
@@ -357,6 +361,84 @@ int Xorriso_interpreter(struct XorrisO *xorriso,
 int Xorriso_execute_option(struct XorrisO *xorriso, char *line, int flag);
 
 
+/* Parse a text line into words. This parsing obeys the same rules as
+   command line parsing but allows to skip a prefix, to use a user provided
+   set of separator characters, and to restrict the number of parsed words.
+
+   If parameter xorriso is NULL, then this call is safe for usage by
+   a concurrent thread while a xorriso API call is being executed.
+
+   @since 1.2.6
+   @param xorriso     The context object which provides settings for parsing
+                      and output channels for error messages.
+                      May be NULL in order to allow concurrent execution e.g.
+                      by a callback function of Xorriso_start_msg_watcher().
+                      If xorriso is NULL then:
+                        flag bit1-bit4 are in effect even if bit0 is not set.
+                        flag bit5 and bit6 may not be set.
+   @param line        A text of one or more words according to man xorriso
+                      paragraph "Command processing" up to and including
+                      "Backslash Interpretation".
+   @param prefix      If not empty then the line will only be parsed if it
+                      begins by the prefix text. Parsing will then begin after
+                      the end of the prefix.
+                      If the prefix does not match, then 0 will be returned
+                      in *argc, argv will be NULL, and the return value will
+                      be 2.
+   @param separators  If not empty this overrides the default list of word
+                      separating characters. Default set is the one of
+                      isspace(3).
+   @param max_words   If not 0: Maximum number of words to parse. If there
+                      remains line text after the last parsed word and its
+                      following separators, then this remainder is copied
+                      unparsed into a final result word. In this case *argc
+                      will be larger than max_words by one. Note that trailing
+                      separators are considered to be followed by an empty
+                      word.
+   @param argc        Will return the number of allocated and filled word
+                      strings.
+   @param argv        Will return the array of word strings.
+                      Do not forget to dispose the allocated memory by a
+                      call to Xorriso__dispose_words().
+   @param flag        Bitfield for control purposes
+                      bit0=   Override setting of -backslash_codes.
+                      bit1-4= With bit0: backslash behavior
+                              0= off
+                              1= in_double_quotes
+                              2= in_quotes
+                              3= with_quoted_input resp. on
+                      bit5=   Prepend the program name as (*argv)[0], so that
+                              *argv is suitable for Xorriso_interpreter()
+                              and other calls which expect this.
+                              Not allowed if xorriso is NULL.
+                      bit6=   Issue failure message in case of return 0
+                              Not allowed if xorriso is NULL.
+   @return            <=0 means error and invalidity of *argv:
+                              0 = Input format error. E.g. bad quotation mark.
+                             -1 = Lack of resources. E.g. memory.
+                             -2 = Improper combination of call parameters.
+                      >0 means success but not necessarily a valid result:
+                              1 = Result in argc and argv is valid (but may
+                                  be empty by argc == 0, argv == NULL).
+                              2 = Line did not match prefix. Result is invalid
+                                  and empty.
+*/
+int Xorriso_parse_line(struct XorrisO *xorriso, char *line,
+                       char *prefix, char *separators, int max_words,
+                       int *argc, char ***argv, int flag);
+
+
+/* Dispose a list of strings as allocated by Xorriso_parse_line() or
+   Xorriso_program_arg_bsl(), or Xorriso_sieve_get_result().
+   @since 1.2.6
+   @param argc        A pointer to the number of allocated and filled word
+                      strings. *argc will be set to 0 by this call.
+   @param argv        A pointer to the array of word strings.
+                      *argv will be set to NULL by this call.
+*/
+void Xorriso__dispose_words(int *argc, char ***argv);
+
+
 /* Enter xorriso command line dialog mode, using libreadline if configured
    at build time and not disabled at run time.
    This call returns immediately if not option -dialog "on" was performed
@@ -442,10 +524,32 @@ int Xorriso_set_problem_status(struct XorrisO *xorriso, char *severity,
                                int flag);
 
 
-/* The next two functions are part of Xorriso_eval_problem_status().
+/* The next three functions are part of Xorriso_eval_problem_status().
    You may use them to build an own advisor function.
 */
 
+/** Compare two severity texts for their severeness.
+    Unknown severity texts get defaulted to "FATAL".
+    @since 1.2.6
+    @param sev1  First severity text to compare
+    @param sev2  Second severity text to compare
+    @return      -1 sev1 is less severe than sev2
+                  0 sev1 is equally severe to sev2
+                  1 sev1 is more severe than sev2
+*/
+int Xorriso__severity_cmp(char *sev1, char *sev2);
+
+
+/** Return a blank separated list of severity names. Sorted from low
+    to high severity.
+    @since 1.2.6
+    @param flag Bitfield for control purposes (unused yet, submit 0)
+    @return  A constant string with the severity names
+*/
+char *Xorriso__severity_list(int flag);
+
+
+
 /** Obtain the current problem status of the xorriso handle.
     @param xorriso    The environment handle
     @param severity   The severity text matching the current problem status
@@ -490,11 +594,379 @@ int Xorriso_process_errfile(struct XorrisO *xorriso,
                             int flag);
 
 
+/*
+                           Message output evaluation
+
+   xorriso is basically a dialog software which reacts on commands by
+   side effects and by messages. The side effects manipulate the state of
+   the ISO image model and of drives. This state can be inquired by commands
+   which emit messages.
+
+   There are several approaches how a program that uses xorriso via this API
+   can receive and use the message output of xorriso.
+
+   - The message sieve may be programmed to pick certain information snippets
+     out of the visible message stream. This covers all messages on the
+     result channel and those info channel messages which get not suppressed
+     by command -report_about. All important info messages have severity NOTE
+     or higher.
+     Much of the message interpretation is supposed to happen by the sieve
+     filter rules which describe the interesting message lines and the
+     positions of the interesting message parts.
+     The call Xorriso_sieve_big() installs a sieve that looks out for most
+     model state messages which xorriso can emit. After a few commands
+     the user will ask the sieve for certain text pieces that might have been
+     caught.
+
+   - The outlist stack may be used to catch messages in linked lists rather
+     than putting them out on the message channels.
+     All interpretation of the messages has to be done by the user of the
+     xorriso API. Function Xorriso_parse_line() is intended to help with
+     splitting up messages into words.
+     The outlist stack is handy for catching the results of information
+     commands with large uniform output or no well recognizable message
+     prefix. Like -lsl, -getfacl, -status, -find ... -exec get_md5.
+     One should push the stack before the command, pull it afterwards, examine
+     the text list by Xorriso_lst_get_*(), and finally dispose the list.
+
+   - The message watcher is a separate program thread which uses the outlist
+     stack to catch the messages and to call user provided handler functions.
+     These functions can use Xorriso_parse_line() too, if they submit the
+     xorriso parameter as NULL. They may not use the struct XorrisO object
+     in any way.
+     Synchronization between watcher and emitters of commands can be achieved
+     by Xorriso_peek_outlists().
+     The main motivation for the message watcher is to inspect and display
+     messages of long lasting xorriso commands while they are still executing.
+     E.g. of -commit, -blank, -format.
+     One would normally start it before such a command and stop it afterwards.
+     But of course, the watcher can stay activated all the time and process
+     all message output via its handler calls.
+
+   The message sieve does not interfere with outlists and message watcher.
+   The message watcher will only see messages which are not caught by outlists
+   which were enabled after the watcher thread was started.
+     
+*/
+
+/* The programmable message sieve picks words out of the program messages
+   of xorriso.
+   The sieve is a collection of filter rules. Each one is defined by a call of
+   Xorriso_sieve_add_filter(). The sieve watches the given output channels for
+   messages which begin by the given text prefixes of the filters.
+   Matching lines get split into words by Xorriso_parse_line() using
+   the given separators. The words described by the filter's word index array
+   get recorded by the filter and can be inquired by Xorriso_sieve_get_result()
+   after one or more xorriso commands have been performed.
+   The recorded results may be disposed by Xorriso_sieve_clear_results without
+   giving up the sieve.
+   The whole sieve may be disposed by Xorriso_sieve_dispose().
+   Default at library start is an inactive sieve without any filter rules.
+*/
+
+/** Add a filter rule to the message sieve.
+    Start watching output messages, if this is not already enabled.
+    @since 1.2.6
+    @param xorriso    The environment handle
+    @param name       The filter name by which its recorded results shall
+                      be inquired via Xorriso_sieve_get_result()
+    @param channels   Which of the output channels the filter shall watch
+                      bit0= result channel
+                      bit1= info channel
+                      bit2= mark channel
+    @param prefix     The line start to watch for. Will also be handed over
+                      to Xorriso_parse_line(). Empty text matches all lines. 
+                      If the prefix begins by '?' characters, then these
+                      match any character at the beginning of a message.
+                      The prefix of the filter rule will then be adapted
+                      to really match the line, before it gets handed over
+                      to Xorriso_parse_line().
+    @param separators List of separator characters for Xorriso_parse_line()
+    @param num_words  Number of word indice in word_idx
+    @param word_idx   Array with the argv indice to be picked from the
+                      the result of Xorriso_parse_line(). Must at least
+                      contain num_words elements.
+    @param max_results If not 0, then the maximum number of line results that
+                      shall be recorded by the filter. When this number is
+                      exceeded, then results of older lines get discarded
+                      when new results get recorded.
+    @param flag       Bitfield for control purposes
+                      bit0= Last result word shall contain the remainder of
+                            the message line
+    @return           <=0 error , >0 success
+*/
+int Xorriso_sieve_add_filter(struct XorrisO *xorriso, char *name,
+                             int channels, char *prefix, char *separators,
+                             int num_words, int *word_idx, int max_results,
+                             int flag);
+
+
+/** Inquire recorded results from a particular filter rule.
+    @param xorriso    The environment handle
+    @param name       The filter name as given by Xorriso_sieve_add_filter()
+    @param argc       Will return the number of allocated and filled word
+                      strings.
+    @param argv       Will return the array of word strings.
+                      Do not forget to dispose the allocated memory by a
+                      call to Xorriso__dispose_words().
+    @param available  Will return the number of results which are still
+                      available for further calls of Xorriso_sieve_get_result()
+                      with the given name.
+    @param flag       Bitfield for control purposes:
+                      bit0= Reset reading to first matching result.
+                      bit1= Only inquire number of available results.
+                            Do not allocate memory.
+                      bit2= If *argv is not NULL, then free it before attaching
+                            new memory.
+                      bit3= Do not read recorded data but rather list all
+                            filter names.
+    @return           <0 error: -1 = memory shortage
+                                -2 = no filter rule found
+                       0 No more data available for the given name
+                         With bit3: No filter rules installed.
+                      >0 argc and argv are valid
+*/
+int Xorriso_sieve_get_result(struct XorrisO *xorriso, char *name,
+                             int *argc, char ***argv, int *available,
+                             int flag);
+
+
+/** Dispose all recorded results. Keep filter rules. Continue watching
+    and recording.
+    @since 1.2.6
+    @param xorriso    The environment handle
+    @param flag       Unused yet. Submit 0.
+    @return           <=0 error , >0 success
+*/
+int Xorriso_sieve_clear_results(struct XorrisO *xorriso, int flag);
+
+
+/** Dispose all filter rules. End watching and recording.
+    This is the default state at library startup.
+    @since 1.2.6
+    @param xorriso    The environment handle
+    @param flag       Unused yet. Submit 0.
+    @return           <=0 error , >0 success
+*/
+int Xorriso_sieve_dispose(struct XorrisO *xorriso, int flag);
+
+
+/** Install a large sieve with filters for about any interesting message
+    of xorriso. The filter rule names are mostly the same as the prefixes they
+    search for. If you do not find the message prefix of your desire, then
+    you may add a filter rule by Xorriso_sieve_add_filter().
+    If you do not want all these filter any more, call Xorriso_sieve_dispose().
+
+    You should obtain your recorded data often and then call
+    Xorriso_sieve_clear_results(). It is nevertheless ok to perform several
+    different xorriso information commands and to then obtain results from the
+    sieve.
+
+    The installed filters in particular:
+    Name             Recorded values, returned by Xorriso_sieve_get_result()
+    ------------------------------------------------------------------------
+    "-changes_pending" up to 1 result from -changes_pending show_status
+                     argv[0]= "yes" or "no"
+    "?  -dev"        up to 10 results from -devices or -device_links
+                     (records drives with single digit index number)
+                     argv[0]= drive address
+                     argv[1]= permissions
+                     argv[2]= drive vendor
+                     argv[3]= product id
+    "??  -dev"       up to 90 results from -devices or -device_links
+                     (records drives with double digit index number)
+                     argv[0]= drive address
+                     argv[1]= permissions
+                     argv[2]= drive vendor
+                     argv[3]= product id
+    "Abstract File:" up to 1 result from -pvd_info
+                     argv[0]= file name
+                     (Note: prefix is "Abstract File: ")
+    "After commit :" up to 1 result from -tell_media_space
+                     argv[0]= number of blocks with "s" appended
+    "App Id       :" up to 1 result from -pvd_info
+                     argv[0]= id
+                     (Note: prefix is "App Id       : ")
+    "Biblio File  :" up to 1 result from -pvd_info
+                     argv[0]= file name
+                     (Note: prefix is "Biblio File  : ")
+    "Build timestamp   :" up to 1 result from -version
+                     argv[0]= timestamp
+                     (Note: prefix is "Build timestamp   :  ")
+    "CopyrightFile:" up to 1 result from -pvd_info
+                     argv[0]= file name
+                     (Note: prefix is "CopyrightFile: ")
+    "Creation Time:" up to 1 result from -pvd_info
+                     argv[0]= YYYYMMDDhhmmsscc
+                     (Note: prefix is "Creation Time: ")
+    "DVD obs 64 kB:" up to 1 result from -list_extras
+                     argv[0]= "yes" or "no"
+    "Drive current:" up to 2 results from -dev, -indev, -toc, others
+                     argv[0]= command ("-dev", "-outdev", "-indev")
+                     argv[1]= drive address
+    "Drive type   :" up to 2 results from -toc
+                     argv[0]= vendor
+                     argv[1]= product
+                     argv[2]= revision
+    "Eff. Time    :" up to 1 result from -pvd_info
+                     argv[0]= YYYYMMDDhhmmsscc
+                     (Note: prefix is "EffectiveTime: ")
+    "Expir. Time  :" up to 1 result from -pvd_info
+                     argv[0]= YYYYMMDDhhmmsscc
+                     (Note: prefix is "Expir. Time  : ")
+    "Ext. filters :" up to 1 result from -list_extras
+                     argv[0]= "yes" or "no" , possibly more info
+                     (Note: prefix is "Ext. filters : ")
+    "File damaged :" up to 10000 results from -find ... -exec report_damage
+                     argv[0]= damage start byte in file
+                     argv[1]= damage range size in file
+                     argv[2]= file size
+                     argv[3]= path in ISO image
+    "File data lba:" up to 10000 results from -find ... -exec report_lba
+                     argv[0]= extent number (all extents of same path together
+                              are the content of one file)
+                     argv[1]= start block number of extent
+                     argv[2]= number of blocks of extent
+                     argv[3]= overall file content size in all extents
+                     argv[4]= path in ISO image
+    "Format idx   :" up to 100 results from -list_formats
+                     argv[0]= index
+                     argv[1]= MMC code
+                     argv[2]= number of blocks with "s" appended
+                     argv[3]= roughly the size in MiB
+                     (Note: prefix is "Format idx ")
+    "Format status:" up to 1 result from -list_formats
+                     argv[0]= status
+                     argv[1]= capacity
+    "ISO session  :" up to 10000 results from -toc
+                     argv[0]= Idx
+                     argv[1]= sbsector
+                     argv[2]= Size
+                     argv[3]= Volume Id
+    "Image size   :" up to 1 result from -print_size
+                     argv[0]= number of blocks with "s" appended
+    "Jigdo files  :" up to 1 result from -list_extras
+                     argv[0]= "yes" or "no"
+    "Local ACL    :" up to 1 result from -list_extras
+                     argv[0]= "yes" or "no"
+    "Local xattr  :" up to 1 result from -list_extras
+                     argv[0]= "yes" or "no"
+    "MD5 MISMATCH:"  up to 10000 results from -check_md5*
+                     argv[0]= path of mismatching file
+    "MD5 tag range:" up to 10000 results from -check_media
+                     argv[0]= lba
+                     argv[1]= size in blocks
+                     argv[2]= result text (starting with "+", "-", or "0")
+    "Media blocks :" up to 2 results from -toc
+                     argv[0]= readable
+                     argv[1]= writable
+                     argv[2]= overall
+    "Media current:" up to 2 results from -dev, -indev, -toc, others
+                     argv[0]= media type / MMC profile name
+                     (Note: prefix is "Media current: " which eats extra blank)
+    "Media nwa    :" up to 1 result from -toc
+                     argv[0]= next writable address
+    "Media product:" up to 2 results from -toc
+                     argv[0]= product id
+                     argv[1]= manufacturer
+    "Media region :" up to 10000 results from -check_media
+                     argv[0]= lba
+                     argv[1]= size in blocks
+                     argv[2]= quality text (starting with "+", "-", or "0")
+    "Media space  :" up to 1 result from -tell_media_space
+                     argv[0]= number of blocks with "s" appended
+    "Media status :" up to 2 results from -dev, -indev, -toc, others
+                     argv[0]= status description
+                     (Note: prefix is "Media status : ")
+    "Media summary:" up to 2 results from -dev, -indev, -toc, others 
+                     argv[0]= sessions
+                     argv[1]= data blocks (full count)
+                     argv[2]= data (with unit letter k,m,g)
+                     argv[3]= free (with unit letter k,m,g)
+    "Modif. Time  :" up to 1 result from -pvd_info
+                     argv[0]= YYYYMMDDhhmmsscc
+                     (Note: prefix is "Modif. Time  : ")
+    "PVD address  :" up to 1 result from -pvd_info
+                     argv[0]= block address with "s" appended
+    "Preparer Id  :" up to 1 result from -pvd_info
+                     argv[0]= id
+                     (Note: prefix is "Preparer Id  : ")
+    "Profile      :" up to 256 results from -list_profiles
+                     argv[0]= MMC code
+                     argv[1]= profile name in round brackets
+                              possibly appended: " (current)"
+    "Publisher Id :" up to 1 result from -pvd_info
+                     argv[0]= id
+                     (Note: prefix is "Publisher Id : ")
+    "Readline     :" up to 1 result from -list_extras
+                     argv[0]= "yes" or "no"
+    "Size lower   :" up to 1 result from -findx ... -exec estimate_size
+                     argv[0]= size with appended "s"
+    "Size upper   :" up to 1 result from -findx ... -exec estimate_size
+                     argv[0]= size with appended "s"
+    "System Id    :" up to 1 result from -pvd_info
+                     argv[0]= id
+                     (Note: prefix is "System Id    : ")
+    "Version timestamp :" up to 1 result from -version
+                     argv[0]= timestamp
+    "Volume Id    :" up to 1 result from -pvd_info
+                     argv[0]= id
+                     (Note: Not output from -dev or -toc but from -pvd_info)
+    "Volume Set Id:" up to 1 result from -pvd_info
+                     argv[0]= id
+                     (Note: prefix is "Volume Set Id: ")
+    "Volume id    :" up to 2 results from -dev, -indev, -toc, others
+                     argv[0]= volume id
+                     (Note: Not output from -pvd_info but from -dev or -toc)
+    "Write speed  :" up to 100 results from -list_speeds
+                     argv[0]= kilobytes per second
+                     argv[1]= speed factor
+    "Write speed H:" up to 1 result from -list_speeds
+                     see "Write speed  :" 
+    "Write speed L:" up to 1 result from -list_speeds
+                     see "Write speed  :" 
+    "Write speed h:" up to 1 result from -list_speeds
+                     see "Write speed  :" 
+    "Write speed l:" up to 1 result from -list_speeds
+                     see "Write speed  :" 
+    "libburn    in use :" up to 1 result from -version
+                     argv[0]= version text
+                     argv[1]= minimum version requirement
+    "libburn OS adapter:" up to 1 result from -version
+                     argv[0]= adapter description
+                     (Note: prefix is "libburn OS adapter:  ")
+    "libisoburn in use :" up to 1 result from -version
+                     argv[0]= version text
+                     argv[1]= minimum version requirement
+    "libisofs   in use :" up to 1 result from -version
+                     argv[0]= version text
+                     argv[1]= minimum version requirement
+    "libjte     in use :" up to 1 result from -version
+                     argv[0]= version text
+                     argv[1]= minimum version requirement
+    "xorriso version   :" up to 1 result from -version
+                     argv[0]= version text
+    "zisofs       :" up to 1 result from -list_extras
+                     argv[0]= "yes" or "no"
+    ------------------------------------------------------------------------
+
+    @since 1.2.6
+    @param xorriso    The environment handle
+    @param flag       Unused yet. Submit 0.
+    @return           <=0 error , >0 success
+*/
+int Xorriso_sieve_big(struct XorrisO *xorriso, int flag);
+
+
 /* The outlist stack allows to redirect the info and result messages from
    their normal channels into a pair of string lists which can at some
    later time be retrieved by the application.
    These redirection caches can be stacked to allow stacked applications.
    xorriso itself uses them for internal purposes. 
+
+   The call Xorriso_start_msg_watcher() starts a concurrent thread which
+   uses outlists to collect messages and to hand them over by calling
+   application provided functions.
 */
 
 /* A list item able of forming double chained lists */
@@ -507,12 +979,14 @@ struct Xorriso_lsT;
     output and eventual older redirections will not see new messages until
     the redirection is ended by a call to Xorriso_pull_outlists() with the
     stack_handle value returned by this call.
-    Redirected output is not written to the files of Xorriso_option_logfile()
-    and the Xorriso_option_pkt_output() protocol will not be applied.
+    If Xorriso_option_pkt_output() is set to "on", then it will consolidate
+    output in the result_list of Xorriso_fetch_outlists() resp.
+    Xorriso_pull_outlists().
     @param xorriso      The environment handle
     @param stack_handle returns an id number which is unique as long as
-                        its redirection is stacked. It may be re-used after
-                        its redirection was pulled from the stack.
+                        its redirection is stacked. Do not interpret it and
+                        do not use it after its redirection was pulled from
+                        the stack.
     @param flag         Bitfield for control purposes
                          bit0= redirect result channel
                          bit1= redirect info channel
@@ -523,6 +997,64 @@ int Xorriso_push_outlists(struct XorrisO *xorriso, int *stack_handle,
                           int flag);
 
 
+/** Obtain the currently collected text messages of redirected info and
+    result channel. 
+    The messages are handed out as two lists. Both lists have to be disposed
+    via Xorriso_lst_destroy_all() when they are no longer needed.
+    The message lists are either NULL or represented by their first
+    Xorriso_lsT item.
+
+    This call is safe for being used by a concurrent thread while a
+    xorriso API call is being executed on the same struct XorrisO.
+    In such a situation, it should not be used with high frequency in order
+    not to hamper the ongoing xorriso operation by blocking its message
+    output facility. A hundred times per second should be enough.
+
+    @since 1.2.6
+    @param xorriso       The environment handle
+    @param stack_handle  An id number returned by Xorriso_push_outlists()
+                         and not yet revoked by Xorriso_pull_outlists().
+                         Submit -1 to address the most recent valid id.
+    @param result_list   Result and mark messages (usually directed to stdout)
+    @param info_list     Info and mark messages (usually directed to stderr)
+    @param flag          Bitfield for control purposes
+                          bit0= fetch result channel
+                          bit1= fetch info channel
+                         If bit0 and bit1 are 0, both channels get fetched.
+    @return              1 on success, <=0 if failure
+*/
+int Xorriso_fetch_outlists(struct XorrisO *xorriso, int stack_handle,
+                           struct Xorriso_lsT **result_list,
+                           struct Xorriso_lsT **info_list, int flag);
+
+
+/** Inquire whether messages are pending in redirected result and info channel.
+    This may be used to determine whether a concurrent message watcher already
+    has obtained all pending messages.
+    @since 1.2.6
+    @param xorriso       The environment handle
+    @param stack_handle  An id number returned by Xorriso_push_outlists()
+                         and not yet revoked by Xorriso_pull_outlists().
+                         Submit -1 to address the most recent valid id.
+    @param timeout       Number of seconds after which to return despite
+                         flag bit 2
+    @param flag          Bitfield for control purposes
+                          bit0= fetch result channel
+                          bit1= fetch info channel
+                          bit2= wait and retry until return is 0 or -1
+                         If bit0 and bit1 are 0, both channels get fetched.
+    @return              If > 0:
+                           bit0= messages are pending in outlists
+                           bit1= message watcher is processing fetched messages
+                         Else:
+                          0= no messages are pending anywhere
+                         -1= inappropriate stack_handle
+                         -2= locking failed
+*/
+int Xorriso_peek_outlists(struct XorrisO *xorriso, int stack_handle,
+                          int timeout, int flag);
+
+
 /** Disable the redirection given by stack_handle. If it was the current
     receiver of messages then switch output to the next older redirection
     resp. to the normal channels if no redirections are stacked any more.
@@ -532,7 +1064,10 @@ int Xorriso_push_outlists(struct XorrisO *xorriso, int *stack_handle,
     The message lists are either NULL or represented by their first
     Xorriso_lsT item.
     @param xorriso       The environment handle
-    @param stack_handle  The id number returned by Xorriso_push_outlists()
+    @param stack_handle  An id number returned by Xorriso_push_outlists()
+                         and not yet revoked by Xorriso_pull_outlists().
+                         This handle is invalid after the call.
+                         Submit -1 to address the most recent valid id.
     @param result_list   Result and mark messages (usually directed to stdout)
     @param info_list     Info and mark messages (usually directed to stderr)
     @param flag          unused yet, submit 0
@@ -543,6 +1078,56 @@ int Xorriso_pull_outlists(struct XorrisO *xorriso, int stack_handle,
                           struct Xorriso_lsT **info_list, int flag);
 
 
+/** Redirect output by Xorriso_push_outlists() and start a thread which
+    fetches this output and performs a call of a given function with each
+    message that is obtained.
+    @since 1.2.6
+    @param xorriso        The environment handle
+    @param result_handler Pointer to the function which shall be called with
+                          each result message. A NULL pointer causes output
+                          to be directed to stdout resp. to be interpreted
+                          as -pkt_output format if this is enabled by
+                          Xorriso_option_pkt_output().
+                          The function should return 1. A return value of -1
+                          urges not to call again with further lines.
+    @param result_handle  The first argument of (*result_handler)(). It shall
+                          point to a memory object that knows all necessary
+                          external parameters for running (*result_handler)().
+                          Submit NULL if result_handler is NULL.
+    @param info_handler   Pointer to the function which shall be called with
+                          each info message. A NULL pointer causes output to
+                          be directed to stderr resp. to -as mkisofs -log-file.
+                          The function should return 1. A return value of -1
+                          urges not to call again with further lines.
+    @param info_handle    The first argument of (*info_handler)(). It shall
+                          point to a memory object that knows all necessary
+                          external parameters for running (*info_handler)().
+                          Submit NULL if info_handler is NULL.
+    @param flag           unused yet, submit 0
+    @return               1 on success, <=0 if failure (e.g. there is already
+                                                        a watcher active)
+*/
+int Xorriso_start_msg_watcher(struct XorrisO *xorriso,
+                    int (*result_handler)(void *handle, char *text),
+                    void *result_handle,
+                    int (*info_handler)(void *handle, char *text),
+                    void *info_handle,
+                    int flag);
+
+
+/** Revoke output redirection by Xorriso_start_msg_watcher() and end the
+    watcher thread. If text messages are delivered when Xorriso_pull_outlists()
+    is called, then they get put out through the active handler functions.
+    @since 1.2.6
+    @param xorriso        The environment handle
+    @param flag           Bitfield for control purposes:
+                          bit0= do not issue SORRY message if no message
+                                watcher is active
+    @return               1 on success, <=0 if failure
+*/
+int Xorriso_stop_msg_watcher(struct XorrisO *xorriso, int flag);
+
+
 /** Obtain the text message from the current list item.
     @param entry    The current list item
     @param flag     unused yet, submit 0
@@ -640,7 +1225,7 @@ int Xorriso_option_add_plainly(struct XorrisO *xorriso, char *mode,
                                    int flag);
 
 
-/* Command -alter_date, alter_date_r */
+/* Command -alter_date, -alter_date_r */
 /* @param flag bit0=recursive (-alter_date_r)
 */
 int Xorriso_option_alter_date(struct XorrisO *xorriso, 
@@ -953,6 +1538,11 @@ int Xorriso_option_jigdo(struct XorrisO *xorriso, char *aspect, char *arg,
 /* Command -joliet "on"|"off" */
 int Xorriso_option_joliet(struct XorrisO *xorriso, char *mode, int flag);
 
+/* Command -launch_frontend */
+/* @since 1.2.6 */
+int Xorriso_option_launch_frontend(struct XorrisO *xorriso,
+                                   int argc, char **argv, int *idx, int flag);
+
 /* Command -list_arg_sorting */
 /* @since 1.2.2 */
 int Xorriso_option_list_arg_sorting(struct XorrisO *xorriso, int flag);
@@ -976,6 +1566,11 @@ int Xorriso_option_list_profiles(struct XorrisO *xorriso, char *which,
 /* @since 1.1.2 */
 int Xorriso_option_list_speeds(struct XorrisO *xorriso, int flag);
 
+/* Command -lns alias -lnsi */
+/* @since 1.2.6 */
+int Xorriso_option_lnsi(struct XorrisO *xorriso, char *target, char *path,
+                        int flag);
+
 /* Command -load session|track|sbsector value */
 /* @param flag bit0= with adr_mode sbsector: adr_value is possibly 16 too high
    @return <=0 error , 1 success, 2 revoked by -reassure
@@ -1044,6 +1639,14 @@ int Xorriso_option_mount(struct XorrisO *xorriso, char *dev, char *adr_mode,
 /* Command -mount_opts option[:...] */
 int Xorriso_option_mount_opts(struct XorrisO *xorriso, char *mode, int flag);
 
+/* Command -move */
+int Xorriso_option_move(struct XorrisO *xorriso, char *origin, char *dest,
+                        int flag);
+
+/* Command -msg_op */
+int Xorriso_option_msg_op(struct XorrisO *xorriso, char *what, char *arg,
+                          int flag);
+
 /* Command -mv alias -mvi */
 int Xorriso_option_mvi(struct XorrisO *xorriso, int argc, char **argv,
                       int *idx, int flag);
@@ -1115,6 +1718,9 @@ int Xorriso_option_path_list(struct XorrisO *xorriso, char *adr, int flag);
 int Xorriso_option_pathspecs(struct XorrisO *xorriso, char *mode, int flag);
 
 /* Command -pkt_output */
+/* Note: If output is redirected by Xorriso_push_outlists() then mode "on"
+         consolidates output in the result output list, not on stdout.
+*/
 int Xorriso_option_pkt_output(struct XorrisO *xorriso, char *mode, int flag);
 
 /* Command -preparer_id */
@@ -1268,8 +1874,13 @@ int Xorriso_option_temp_mem_limit(struct XorrisO *xorriso, char *size,
                                   int flag);
 
 /* Command -toc */
+/* @param flag   bit0= short report form as with -dev, no table-of-content
+*/
 int Xorriso_option_toc(struct XorrisO *xorriso, int flag);
 
+/* Command -toc_of */
+int Xorriso_option_toc_of(struct XorrisO *xorriso, char *which, int flag);
+
 /* Command -uid */
 int Xorriso_option_uid(struct XorrisO *xorriso, char *uid, int flag);
 
