@@ -2,7 +2,7 @@
 /* Command line oriented batch and dialog tool which creates, loads,
    manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -19,6 +19,12 @@
 #define Xorriso_private_includeD yes
 
 
+/* <<< Disable this to disable pthread_mutex locking on message and result
+       output.
+*/
+#define Xorriso_fetch_with_msg_queueS yes
+
+
 /* for uint32_t */
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
@@ -132,7 +138,11 @@ struct XorrisO { /* the global context of xorriso */
                     */
 
  int iso_level;
+ int iso_level_is_default;
  int do_joliet;
+ int do_hfsplus;
+ int do_fat;
+ int do_rockridge;
  int do_iso1999;
 
  int do_aaip; /* bit0= ACL in
@@ -170,6 +180,7 @@ struct XorrisO { /* the global context of xorriso */
  char scdbackup_tag_listname[SfileadrL];
 
  int relax_compliance;        /* opaque bitfield to be set by xorrisoburn */
+ int allow_dir_id_ext_dflt; /* -compliance allow_dir_id_ext still on default */
  char rr_reloc_dir[256];
  int rr_reloc_flags;
  int untranslated_name_len;
@@ -186,6 +197,10 @@ struct XorrisO { /* the global context of xorriso */
  mode_t global_dir_mode;
  mode_t global_file_mode;
 
+ int do_tao; /* 1= Use TAO resp. Incremental
+               -1= Use SAO resp. DAO
+                0= let libburn choose */
+
  struct Xorriso_lsT *filters;
  int filter_list_closed;
 
@@ -231,6 +246,8 @@ struct XorrisO { /* the global context of xorriso */
                                   do not emulate TOC on overwriteable media
                             bit2= bit7 for isoburn_drive_aquire()
                                   pretend any media to be -ROM
+                            bit3= bit9 for isoburn_drive_aquire()
+                                  Ignore enclosing session at LBA 0
                          */
 
  int image_start_mode;  /* From what address to load the ISO image
@@ -336,17 +353,23 @@ struct XorrisO { /* the global context of xorriso */
 
  int keep_boot_image;
  char boot_image_cat_path[SfileadrL];
- int boot_image_cat_hidden;   /* bit0= hidden in ISO/RR , bit1= in Joliet */
+ int boot_image_cat_hidden;   /* bit0= hidden in ISO/RR , bit1= in Joliet ,
+                                 bit2= in HFS+ */
  int boot_count; /* number of already attached boot images */
 
  char boot_image_bin_path[SfileadrL];
  char boot_image_bin_form[16];
  int boot_platform_id; 
- int patch_isolinux_image; /* bit0= boot-info-table , bit1= not with EFI */
+ int patch_isolinux_image; /* bit0= boot-info-table , bit1= not with EFI
+                              bit2-7= Mentioning in isohybrid GPT
+                                      1=EFI, 2=HFS+
+                              bit8= Mention in isohybrid Apple Partition Map
+                            */
  int boot_image_emul; /* 0=no emulation
                          1=emulation as hard disk
                          2=emulation as floppy
                       */
+ int boot_emul_default;  /* 1= boot_image_emul is still default */
  off_t boot_image_load_size;
  unsigned char boot_id_string[29];
  unsigned char boot_selection_crit[21];
@@ -372,6 +395,16 @@ struct XorrisO { /* the global context of xorriso */
                                                0 = auto (align if bit1)
                                                1 = always align
                                                2 = never align
+                                         bit10-13= System area sub type
+                                               With type 0 = MBR:
+                                               Gets overridden by bit0 and bit1.
+                                               0 = no particular sub type
+                                               1 = CHRP: A single MBR partition
+                                                   of type 0x96 covers the ISO
+                                                   image. Not compatible with 
+                                                   any other feature which
+                                                   needs to have own MBR
+                                                   partition entries.
                                        */
  int patch_system_area;               /* Bits as of system_area_options.
                                          to be applied to the loaded system
@@ -389,6 +422,10 @@ struct XorrisO { /* the global context of xorriso */
  /* 1 to 255, 0= disabled/default */
  int partition_heads_per_cyl;
 
+ /* Disk file paths of content of PreP partition and EFI system partition */
+ char prep_partition[SfileadrL];
+ char efi_boot_partition[SfileadrL];
+
  /* Path and type of image files to be appended as MBR partitions */
  char *appended_partitions[Xorriso_max_appended_partitionS];
  uint8_t appended_part_types[Xorriso_max_appended_partitionS];
@@ -397,6 +434,16 @@ struct XorrisO { /* the global context of xorriso */
  */
  char ascii_disc_label[Xorriso_disc_label_sizE];
 
+ /* HFS+ image serial number.
+    00...00 means that it shall be generated by libisofs.
+ */
+ uint8_t hfsp_serial_number[8];
+
+ /* Allocation block size of HFS+  and APM : 0= auto , 512, or 2048
+ */
+ int hfsp_block_size;
+ int apm_block_size;
+
  /* User settable PVD time stamps */
  time_t vol_creation_time;
  time_t vol_modification_time;
@@ -429,7 +476,8 @@ struct XorrisO { /* the global context of xorriso */
  /* XORRISO options */
  int allow_graft_points;
 
- int allow_restore; /* -1=permanently disallowed
+ int allow_restore; /* -2=disallowed until special mode "unblock"
+                       -1=permanently disallowed
                         0=disallowed, 1=allowed, 2=device files allowed */
  int do_concat_split;  /* 1= restore complete split file directories as
                              regular files
@@ -505,6 +553,7 @@ struct XorrisO { /* the global context of xorriso */
 
  struct ExclusionS *iso_rr_hidings;
  struct ExclusionS *joliet_hidings;
+ struct ExclusionS *hfsplus_hidings;
 
  int use_stdin; /* use raw stdin even if readline support is compiled */
  int result_page_length;
@@ -514,6 +563,7 @@ struct XorrisO { /* the global context of xorriso */
  char logfile[4][SfileadrL];
  FILE *logfile_fp[4];
  FILE *pktlog_fp;
+ FILE *stderr_fp;
  struct Xorriso_lsT *result_msglists[Xorriso_max_outlist_stacK];
  struct Xorriso_lsT *info_msglists[Xorriso_max_outlist_stacK];
  int msglist_flags[Xorriso_max_outlist_stacK]; /* bit0= result is redirected
@@ -521,6 +571,33 @@ struct XorrisO { /* the global context of xorriso */
                                                */
  int msglist_stackfill;
 
+ int lib_msg_queue_lock_ini;
+ int result_msglists_lock_ini;
+ pthread_mutex_t lib_msg_queue_lock;
+ pthread_mutex_t result_msglists_lock;
+
+ int write_to_channel_lock_ini;
+ pthread_mutex_t write_to_channel_lock;
+
+ int msg_watcher_lock_ini;
+ pthread_mutex_t msg_watcher_lock;
+ int msg_watcher_state; /* 0= inactive
+                           1= registered
+                           2= started
+                           3= request to end
+                        */
+ int (*msgw_result_handler)(void *handle, char *text);
+ void *msgw_result_handle;
+ int (*msgw_info_handler)(void *handle, char *text);
+ void *msgw_info_handle;
+ int msgw_stack_handle;
+ int msgw_msg_pending; /* 0=no, 1=fetching(i.e. maybe) , 2=yes */
+ int msgw_fetch_lock_ini;
+ pthread_mutex_t msgw_fetch_lock;
+
+ struct Xorriso_msg_sievE *msg_sieve;
+ int msg_sieve_disabled;
+
  int status_history_max; /* for -status long_history */
  
  /* 0= no logging of SCSI commands, 1= to stderr */
@@ -534,6 +611,8 @@ struct XorrisO { /* the global context of xorriso */
  int abort_on_is_default; /* will be set to 0 by first -abort_on */
  int problem_status; /* Severity rank number. 0= no abort condition present */
  char problem_status_text[20];
+ int problem_status_lock_ini;
+ pthread_mutex_t problem_status_lock;
 
  char errfile_log[SfileadrL]; /* for -errfile_log */
  int errfile_mode; /* bit0= marked */
@@ -569,6 +648,7 @@ struct XorrisO { /* the global context of xorriso */
  double insert_count;
  double insert_bytes;
  double error_count; /* double will not roll over */
+ int launch_frontend_banned;
 
  /* pacifiers */
  int pacifier_style; /* 0= xorriso, 1=mkisofs 2=cdrecord */
