@@ -4,7 +4,7 @@
    a command line oriented batch and dialog tool which creates, loads,
    manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -19,11 +19,12 @@
 */
 #define xorriso_libisoburn_req_major  1
 #define xorriso_libisoburn_req_minor  2
-#define xorriso_libisoburn_req_micro  4
+#define xorriso_libisoburn_req_micro  8
 
 
 struct SpotlisT;          /* List of intervals with different read qualities */
 struct CheckmediajoB;     /* Parameters for Xorriso_check_media() */
+struct Xorriso_msg_sievE; /* Fishes for info particles in reply messages */
 
 
 int Xorriso_startup_libraries(struct XorrisO *xorriso, int flag);
@@ -605,5 +606,7 @@ int Xorriso_hfsplus_file_creator_type(struct XorrisO *xorriso, char *path,
 int Xorriso_hfsplus_bless(struct XorrisO *xorriso, char *path,
                           void *in_node, char *blessing, int flag);
 
+int Xorriso_pretend_full_disc(struct XorrisO *xorriso, int flag);
+
 #endif /* Xorrisoburn_includeD */
 
